import { ButtonGroup } from "./group.button";
import { LoadingBar } from "../single";
import {
    VerticalLayout,
    shortcut,
    Widget,
    Controller,
    extend,
    createWidget,
    isEmpty,
    nextTick,
    bind,
    isFunction,
    isNotEmptyArray,
    isNumber,
    isObject,
    each,
    emptyFn,
    Events,
    LogicFactory
} from "@/core";

/**
 * 加载控件
 *
 * Created by GUY on 2015/8/31.
 * @class Loader
 * @extends Widget
 */

@shortcut()
export class Loader extends Widget {
    static xtype = "bi.loader";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-loader",

            direction: "top",
            isDefaultInit: true, // 是否默认初始化数据
            logic: {
                dynamic: true,
                scrolly: true,
            },

            // 下面是button_group的属性
            el: {
                type: ButtonGroup.xtype,
            },

            items: [],
            itemsCreator: emptyFn,
            onLoaded: emptyFn,

            // 下面是分页信息
            count: false,
            prev: false,
            next: {},
            hasPrev: emptyFn,
            hasNext: emptyFn,
        });
    }

    _prevLoad() {
        const o = this.options;
        this.prev.setLoading();
        o.itemsCreator.apply(this, [
            { times: --this.times },
            (...args) => {
                this.prev.setLoaded();
                this.prependItems(...args);
            }
        ]);
    }

    _nextLoad() {
        const o = this.options;
        this.next.setLoading();
        o.itemsCreator.apply(this, [
            { times: ++this.times },
            (...args) => {
                this.next.setLoaded();
                this.addItems(...args);
            }
        ]);
    }

    render() {
        const {
            itemsCreator,
            el,
            items: optionsItems,
            value,
            direction,
            logic,
            isDefaultInit,
        } = this.options;
        if (itemsCreator === false) {
            this.options.prev = false;
            this.options.next = false;
        }
        if (this.options.prev !== false) {
            this.prev = createWidget(
                extend(
                    {
                        type: LoadingBar.xtype,
                    },
                    this.options.prev
                )
            );
            this.prev.on(Controller.EVENT_CHANGE, type => {
                if (type === Events.CLICK) {
                    this._prevLoad();
                }
            });
        }

        this.button_group = createWidget(el, {
            type: ButtonGroup.xtype,
            chooseType: 0,
            items: optionsItems,
            behaviors: {},
            layouts: [
                {
                    type: VerticalLayout.xtype,
                }
            ],
            value,
        });
        this.button_group.on(Controller.EVENT_CHANGE, (type, value, obj, ...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, type, value, obj, ...args);
            if (type === Events.CLICK) {
                this.fireEvent(Loader.EVENT_CHANGE, obj);
            }
        });

        if (this.options.next !== false) {
            this.next = createWidget(
                extend(
                    {
                        type: LoadingBar.xtype,
                    },
                    this.options.next
                )
            );
            this.next.on(Controller.EVENT_CHANGE, type => {
                if (type === Events.CLICK) {
                    this._nextLoad();
                }
            });
        }

        createWidget(
            extend(
                {
                    element: this,
                },
                LogicFactory.createLogic(
                    LogicFactory.createLogicTypeByDirection(direction),
                    extend(
                        {
                            scrolly: true,
                        },
                        logic,
                        {
                            items: LogicFactory.createLogicItemsByDirection(
                                direction,
                                this.prev,
                                this.button_group,
                                this.next
                            ),
                        }
                    )
                )
            )
        );

        isDefaultInit &&
            isEmpty(optionsItems) &&
            nextTick(
                bind(() => {
                    isDefaultInit && isEmpty(this.options.items) && this._populate();
                }, this)
            );
        const items = isFunction(optionsItems)
            ? this.__watch(optionsItems, (context, newValue) => {
                this.populate(newValue);
            })
            : optionsItems;
        if (isNotEmptyArray(items)) {
            this._populate(items);
        }
    }

    hasPrev() {
        const { count, hasPrev } = this.options;
        if (isNumber(count)) {
            return this.count < count;
        }

        return !!hasPrev.apply(this, [
            {
                times: this.times,
                count: this.count,
            }, this.options
        ]);
    }

    hasNext() {
        const { count, hasNext } = this.options;
        if (isNumber(count)) {
            return this.count < count;
        }

        return !!hasNext.apply(this, [
            {
                times: this.times,
                count: this.count,
            }, this.options
        ]);
    }

    prependItems(items) {
        this.count += items.length;
        if (this.next !== false) {
            if (this.hasPrev()) {
                this.options.items = this.options.items.concat(items);
                this.prev.setLoaded();
            } else {
                this.prev.setEnd();
            }
        }
        this.button_group.prependItems(...arguments);
    }

    addItems(items) {
        this.count += items.length;
        if (isObject(this.next)) {
            if (this.hasNext()) {
                this.options.items = this.options.items.concat(items);
                this.next.setLoaded();
            } else {
                this.next.setEnd();
            }
        }
        this.button_group.addItems(...arguments);
    }

    _populate(items) {
        const o = this.options;
        if (arguments.length === 0 && isFunction(o.itemsCreator)) {
            o.itemsCreator.apply(this, [
                { times: 1 },
                (...args) => {
                    if (args.length === 0) {
                        throw new Error("Parameter cannot be empty");
                    }
                    this.populate(...args);
                    o.onLoaded();
                }
            ]);

            return false;
        }
        this.options.items = items;
        this.times = 1;
        this.count = 0;
        this.count += items.length;
        if (isObject(this.next)) {
            if (this.hasNext()) {
                this.next.setLoaded();
            } else {
                this.next.invisible();
            }
        }
        if (isObject(this.prev)) {
            if (this.hasPrev()) {
                this.prev.setLoaded();
            } else {
                this.prev.invisible();
            }
        }

        return true;
    }

    populate() {
        this._populate(...arguments) && this.button_group.populate(...arguments);
    }

    setNotSelectedValue() {
        this.button_group.setNotSelectedValue(...arguments);
    }

    getNotSelectedValue() {
        return this.button_group.getNotSelectedValue();
    }

    setValue() {
        this.button_group.setValue(...arguments);
    }

    getValue() {
        return this.button_group.getValue(...arguments);
    }

    getAllButtons() {
        return this.button_group.getAllButtons();
    }

    getAllLeaves() {
        return this.button_group.getAllLeaves();
    }

    getSelectedButtons() {
        return this.button_group.getSelectedButtons();
    }

    getNotSelectedButtons() {
        return this.button_group.getNotSelectedButtons();
    }

    getIndexByValue(value) {
        return this.button_group.getIndexByValue(value);
    }

    getNodeById(id) {
        return this.button_group.getNodeById(id);
    }

    getNodeByValue(value) {
        return this.button_group.getNodeByValue(value);
    }

    empty() {
        this.button_group.empty();
        each([this.prev, this.next], (i, ob) => {
            ob && ob.setVisible(false);
        });
    }

    destroy() {
        super.destroy(...arguments);
    }
}
