import { VerticalLayout, Layout, Widget, shortcut, isFunction, each, PrefixIntervalTree, ResizeDetector } from "@/core";

/**
 * 虚拟列表
 *
 * Created by GUY on 2017/5/22.
 * @class VirtualList
 * @extends Widget
 */

@shortcut()
export class VirtualList extends Widget {
    props() {
        return {
            baseCls: "bi-virtual-list",
            overscanHeight: 100,
            blockSize: 10,
            scrollTop: 0,
            items: [],
            itemFormatter: (item, index) => item,
        };
    }

    init() {
        this.renderedIndex = -1;
        this.cache = {};
    }

    static xtype = "bi.virtual_list";

    render() {
        return {
            type: VerticalLayout.xtype,
            items: [
                {
                    type: Layout.xtype,
                    ref: _ref => {
                        this.topBlank = _ref;
                    },
                },
                {
                    type: VerticalLayout.xtype,
                    scrolly: false,
                    ref: _ref => {
                        this.container = _ref;
                    },
                },
                {
                    type: Layout.xtype,
                    ref: _ref => {
                        this.bottomBlank = _ref;
                    },
                }
            ],
        };
    }
    // mounted之后绑定事件
    mounted() {
        //  这里无法进行结构，因为存在赋值操作，如果使用结构则this.options的值不会跟随变化
        const o = this.options;
        o.items = isFunction(o.items)
            ? this.__watch(o.items, (context, newValue) => {
                this.populate(newValue);
            })
            : o.items;
        this._populate();
        this.element.scroll(e => {
            o.scrollTop = this.element.scrollTop();
            this._calculateBlocksToRender();
        });
        ResizeDetector.addResizeListener(this, () => {
            if (this.element.is(":visible")) {
                this._calculateBlocksToRender();
            }
        });
    }

    _renderMoreIf() {
        const { scrollTop, overscanHeight, blockSize, items, itemFormatter } = this.options;
        const height = this.element.height();
        const minContentHeight = scrollTop + height + overscanHeight;
        let index = (this.renderedIndex + 1) * blockSize,
            cnt = this.renderedIndex + 1;
        let lastHeight;
        const getElementHeight = () =>
            this.container.element.height() + this.topBlank.element.height() + this.bottomBlank.element.height();
        lastHeight = getElementHeight();
        while (lastHeight < minContentHeight && index < items.length) {
            const itemsArr = items.slice(index, index + blockSize);
            this.container.addItems(
                // eslint-disable-next-line no-loop-func
                itemsArr.map((item, i) => itemFormatter(item, index + i)),
                this
            );
            const addedHeight = getElementHeight() - lastHeight;
            this.tree.set(cnt, addedHeight);
            this.renderedIndex = cnt;
            cnt++;
            index += blockSize;
            lastHeight = getElementHeight();
        }
    }

    _calculateBlocksToRender() {
        const { scrollTop, overscanHeight, blockSize, items, itemFormatter } = this.options;
        // BI-115750 不可见状态下依赖元素实际尺寸构造的线段树会分段错误，所以不进行后续计算和线段树的初始化。
        // 这样从不可见状态变为可见状态能够重新触发线段树初始化
        if (!this.element.is(":visible")) {
            return;
        }
        this._renderMoreIf();
        const height = this.element.height();
        const minContentHeightFrom = scrollTop - overscanHeight;
        const minContentHeightTo = scrollTop + height + overscanHeight;
        const start = this.tree.greatestLowerBound(minContentHeightFrom);
        const end = this.tree.leastUpperBound(minContentHeightTo);
        const needDestroyed = [],
            needMount = [];
        for (let i = 0; i < start; i++) {
            const index = i * blockSize;
            if (!this.cache[i]) {
                this.cache[i] = {};
            }
            if (!this.cache[i].destroyed) {
                for (let j = index; j < index + blockSize && j < items.length; j++) {
                    needDestroyed.push(this.container._children[j]);
                    this.container._children[j] = null;
                }
                this.cache[i].destroyed = true;
            }
        }
        for (let i = end + 1; i <= this.renderedIndex; i++) {
            const index = i * blockSize;
            if (!this.cache[i]) {
                this.cache[i] = {};
            }
            if (!this.cache[i].destroyed) {
                for (let j = index; j < index + blockSize && j < items.length; j++) {
                    needDestroyed.push(this.container._children[j]);
                    this.container._children[j] = null;
                }
                this.cache[i].destroyed = true;
            }
        }
        const firstFragment = Widget._renderEngine.createFragment(),
            lastFragment = Widget._renderEngine.createFragment();
        let currentFragment = firstFragment;
        for (let i = start < 0 ? 0 : start; i <= end && i <= this.renderedIndex; i++) {
            const index = i * blockSize;
            if (!this.cache[i]) {
                this.cache[i] = {};
            }
            if (!this.cache[i].destroyed) {
                currentFragment = lastFragment;
            }
            if (this.cache[i].destroyed === true) {
                for (let j = index; j < index + blockSize && j < items.length; j++) {
                    const w = this.container._addElement(j, itemFormatter(items[j], j), this);
                    needMount.push(w);
                    currentFragment.appendChild(w.element[0]);
                }
                this.cache[i].destroyed = false;
            }
        }
        this.container.element.prepend(firstFragment);
        this.container.element.append(lastFragment);
        this.topBlank.setHeight(`${this.tree.sumTo(Math.max(-1, start - 1))}px`);
        this.bottomBlank.setHeight(
            `${this.tree.sumTo(this.renderedIndex) - this.tree.sumTo(Math.min(end, this.renderedIndex))}px`
        );
        each(needMount, (i, child) => {
            child && child._mount();
        });
        each(needDestroyed, (i, child) => {
            child && child._destroy();
        });
    }
    _populate(items) {
        const { blockSize, scrollTop } = this.options;
        if (items && this.options.items !== items) {
            this.options.items = items;
        }
        this.tree = PrefixIntervalTree.empty(Math.ceil(this.options.items.length / blockSize));

        this._calculateBlocksToRender();
        try {
            this.element.scrollTop(scrollTop);
        } catch (e) {}
    }

    _clearChildren() {
        each(this.container._children, (i, cell) => {
            cell && cell._destroy();
        });
        this.container._children = {};
        this.container.attr("items", []);
    }

    scrollTo(scrollTop) {
        this.options.scrollTop = scrollTop;
        this._calculateBlocksToRender();
        this.element.scrollTop(scrollTop);
    }

    restore() {
        this.renderedIndex = -1;
        this._clearChildren();
        this.cache = {};
        this.options.scrollTop = 0;
        // 依赖于cache的占位元素也要初始化
        this.topBlank.setHeight(0);
        this.bottomBlank.setHeight(0);
    }

    populate(items) {
        if (items && this.options.items !== items) {
            this.restore();
        }
        this._populate(items);
    }

    beforeDestroy() {
        ResizeDetector.removeResizeListener(this);
        this.restore();
    }
}
