import { CenterAdaptLayout, isNumber, shortcut, isPlainObject, createWidget, isIE, isIE9Below, pixFormat, get, isNull, toPix } from "@/core";
import { Label, IconLabel } from "../../label";
import { BasicButton } from "../button.basic";

function isVertical(position) {
    return position === "top" || position === "bottom";
}

const loadingCls = "button-loading-font anim-rotate";

/**
 * 文字类型的按钮
 * @class Button
 * @extends BasicButton
 *
 * @cfg {JSON} options 配置属性
 * @cfg {'common' / 'success' / 'warning' / 'ignore'} [options.level='common'] 按钮类型，用不同颜色强调不同的场景
 */
@shortcut()
export class Button extends BasicButton {
    _const = {
        iconWidth: 18,
    };

    static xtype = "bi.button";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig(props) {
        const conf = super._defaultConfig.apply(this, arguments);

        let adaptiveHeight = 0;
        if (isVertical(props.iconPosition)) {
            // 图标高度和文字高度默认相等
            adaptiveHeight += (props.textHeight || 16) * 2;
            adaptiveHeight += props.iconGap || 0;
            const tGap = props.tgap || props.vgap || 2;
            const bGap = props.bgap || props.vgap || 2;
            adaptiveHeight += tGap + bGap;
        }

        const clearMinWidth = props.block === true || props.clear === true || props.plain;

        return {
            ...conf,
            baseCls: `${conf.baseCls || ""} bi-button${isIE() && isIE9Below() ? " hack" : ""}`,
            attributes: {
                tabIndex: 1,
            },
            minWidth: clearMinWidth ? 0 : 80,
            height: isVertical(props.iconPosition) ? adaptiveHeight : 24,
            shadow: props.clear !== true,
            isShadowShowingOnSelected: true,
            readonly: true,
            iconCls: "",
            level: "common",
            block: false, // 是否块状显示，即不显示边框，没有最小宽度的限制
            clear: false, // 是否去掉边框和背景
            ghost: false, // 是否幽灵显示, 即正常状态无背景
            loading: false, // 是否处于加载中
            light: false, // 是否使用浅色
            plain: false, // 是否是朴素按钮，和 clear 的区别是 plain 有悬浮效果
            textAlign: "center",
            whiteSpace: "nowrap",
            textWidth: null,
            textHeight: null,
            hgap: props.clear ? 0 : props.plain && !props.text ? 4 : 10,
            vgap: 0,
            tgap: 0,
            bgap: 0,
            lgap: 0,
            rgap: 0,
            icon: "",
            iconGap: 0,
            iconPosition: "left",
        };
    }

    render() {
        const o = this.options;

        // bi.center_adapt 作用：让 hgap 不影响 iconGap。
        createWidget({
            type: CenterAdaptLayout.xtype,
            horizontalAlign: o.textAlign,
            element: this,
            ref: ref => {
                this.containerRef = ref;
            },
            hgap: o.hgap,
            vgap: o.vgap,
            items: this.generateItems(),
        });

        // 如果 options 对应的属性为 true 则给元素添加 class
        const classArr = ["block", "clear", "ghost", "plain", "loading", "light"];
        classArr.forEach(clz => {
            if (get(o, clz) === true) {
                this.element.addClass(clz);
            }
        });

        if (o.minWidth > 0) {
            this.element.css({ "min-width": pixFormat(o.minWidth) });
        }
    }

    generateItems(defaultRenderIcon) {
        const o = this.options;

        // 由于button默认情况下有个边框，所以要主动算行高
        let lineHeight,
            textHeight = o.textHeight;
        let hasBorder = false;
        if (isNumber(o.height)) {
            if (!isVertical(o.iconPosition)) {
                if (!(o.clear && o.block && o.light)) {
                    hasBorder = true;
                }
                lineHeight = o.height;
            } else {
                lineHeight = textHeight;
            }
        }
        if (!textHeight) {
            if (o.whiteSpace === "nowrap") {
                textHeight = lineHeight;
            }
        }

        const iconInvisible = !(o.loading || o.iconCls || o.icon || defaultRenderIcon);

        let maxTextWidth = Math.max(o.minWidth, o.width);
        maxTextWidth -= o.hgap * 2 + o.iconGap;
        // 减去图标水平占位宽度
        maxTextWidth -= iconInvisible || isVertical(o.iconPosition) ? 0 : this._const.iconWidth;
        const textWidth = isNull(o.textWidth) ? maxTextWidth : Math.min(o.textWidth, maxTextWidth);

        this.text = createWidget({
            type: Label.xtype,
            text: o.text,
            whiteSpace: o.whiteSpace,
            textAlign: o.textAlign,
            textWidth,
            textHeight: toPix(textHeight, hasBorder ? 2 : 0),
            height: toPix(lineHeight, hasBorder ? 2 : 0),
            value: o.value,
            title: null,
        });

        if (iconInvisible) {
            return [this.text];
        }

        this._iconRendered = true;

        if (isPlainObject(o.icon) && !o.loading) {
            this.icon = createWidget(o.icon);
        } else {
            this.icon = createWidget({
                type: IconLabel.xtype,
                cls: o.loading ? loadingCls : o.iconCls || o.icon,
                width: this._const.iconWidth,
                height: toPix(lineHeight, hasBorder ? 2 : 0),
                lineHeight: toPix(lineHeight, hasBorder ? 2 : 0),
                // 不设置，自定义按钮无法居中
                iconWidth: o.iconWidth,
                iconHeight: o.iconHeight,
                invisible: iconInvisible,
            });
        }

        const gapContainer = {
            lgap: o.iconPosition === "left" && o.text ? o.iconGap : 0,
            rgap: o.iconPosition === "right" ? o.iconGap : 0,
            tgap: o.iconPosition === "top" ? o.iconGap : 0,
            bgap: o.iconPosition === "bottom" ? o.iconGap : 0,
        };

        const items = [this.icon, { el: this.text, ...gapContainer }];
        if (o.iconPosition === "right" || o.iconPosition === "bottom") {
            items.reverse();
        }

        return [
            {
                type: isVertical(o.iconPosition) ? "bi.vertical" : "bi.horizontal",
                horizontalAlign: "center",
                verticalAlign: "middle",
                items,
            }
        ];
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(Button.EVENT_CHANGE, this);
        }
    }

    _setEnable(enable) {
        super._setEnable(...arguments);
        if (enable === true) {
            this.element.attr("tabIndex", 1);
        } else if (enable === false) {
            this.element.removeAttr("tabIndex");
        }
    }

    beforeClick() {
        return this.isLoading();
    }

    isLoading() {
        return this._loading === undefined ? this.options.loading : this._loading;
    }

    loading() {
        this._loading = true;
        this.element.addClass("loading");
        !this._iconRendered && this.containerRef.populate(this.generateItems(true));
        if (this.icon.loading) {
            this.icon.loading();
        } else {
            // loadingCls 可以覆盖 iconCls 所以不需要移除 iconCls
            this.icon.element.addClass(loadingCls);
            this.icon.setVisible(true);
        }
    }

    loaded() {
        this._loading = false;
        this.element.removeClass("loading");
        if (this.icon.loaded) {
            this.icon.loaded();
        } else {
            this.icon.element.removeClass(loadingCls);
            this.icon.setVisible(!!this.options.iconCls);
        }
    }

    setText(text) {
        super.setText(...arguments);
        this.text.setText(text);
    }

    setValue(text) {
        super.setValue(...arguments);
        if (!this.isReadOnly()) {
            this.text.setValue(text);
        }
    }

    setIcon(cls) {
        const o = this.options;
        !this._iconRendered && this.containerRef.populate(this.generateItems(true));
        if (this.icon && o.iconCls !== cls) {
            this.icon.element.removeClass(o.iconCls).addClass(cls);
            o.iconCls = cls;
        }
    }

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doHighLight() {
        this.text.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }
}
