import { ButtonGroup, TextItem } from "@/base";
import {
    CenterAdaptLayout,
    shortcut,
    Widget,
    extend,
    parseDateTime,
    range,
    checkDateVoid,
    print,
    getDate,
    each,
    createWidget,
    createItems,
    LogicFactory,
    Controller,
    makeArray,
    map,
    isNotEmptyString,
    SIZE_CONSANTS
} from "@/core";

/**
 * Created by GUY on 2015/8/28.
 * @class YearCalendar
 * @extends Widget
 */
@shortcut()
export class YearCalendar extends Widget {
    static xtype = "bi.year_calendar";
    static INTERVAL = 12;

    // 获取显示的第一年
    static getStartYear(year) {
        const cur = getDate().getFullYear();

        return year - ((((year - cur + 3) % YearCalendar.INTERVAL) + 12) % YearCalendar.INTERVAL);
    }

    static getEndYear(year) {
        return YearCalendar.getStartYear(year) + YearCalendar.INTERVAL - 1;
    }

    static getPageByYear(year) {
        const cur = getDate().getFullYear();
        year = YearCalendar.getStartYear(year);

        return (year - cur + 3) / YearCalendar.INTERVAL;
    }

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-year-calendar",
            behaviors: {},
            logic: {
                dynamic: false,
            },
            min: "1900-01-01", // 最小日期
            max: "2099-12-31", // 最大日期
            year: null,
        });
    }

    _yearCreator(Y) {
        const { min, max } = this.options;
        Y = Y | 0;
        const start = YearCalendar.getStartYear(Y);
        const items = [];
        // 对于年控件来说，只要传入的minDate和maxDate的year区间包含v就是合法的
        const startDate = parseDateTime(min, "%Y-%X-%d");
        const endDate = parseDateTime(max, "%Y-%X-%d");
        each(range(YearCalendar.INTERVAL), i => {
            const td = {};
            if (
                checkDateVoid(
                    start + i,
                    1,
                    1,
                    print(getDate(startDate.getFullYear(), 0, 1), "%Y-%X-%d"),
                    print(getDate(endDate.getFullYear(), 0, 1), "%Y-%X-%d")
                )[0]
            ) {
                td.disabled = true;
            }
            td.text = start + i;
            items.push(td);
        });

        return items;
    }
    _init() {
        super._init(...arguments);
        const { behaviors, logic } = this.options;
        this.currentYear = getDate().getFullYear();

        this.years = createWidget({
            type: ButtonGroup.xtype,
            behaviors,
            items: createItems(this._getItems(), {}),
            layouts: [
                LogicFactory.createLogic(
                    "table",
                    extend({}, logic, {
                        columns: 2,
                        rows: 6,
                        columnSize: [1 / 2, 1 / 2],
                        rowSize: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
                    })
                ),
                {
                    type: CenterAdaptLayout.xtype,
                    vgap: 2,
                }
            ],
        });
        this.years.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        createWidget(
            extend(
                {
                    element: this,
                },
                LogicFactory.createLogic(
                    "vertical",
                    extend({}, logic, {
                        scrolly: true,
                        vgap: 5,
                        hgap: 6,
                        items: LogicFactory.createLogicItemsByDirection("top", this.years),
                    })
                )
            )
        );
    }

    isFrontYear() {
        const { min, max } = this.options;
        let Y = this.options.year;
        Y = Y | 0;

        return !!checkDateVoid(YearCalendar.getStartYear(Y) - 1, 1, 1, min, max)[0];
    }

    isFinalYear() {
        const { min, max } = this.options;
        let Y = this.options.year;
        Y = Y | 0;

        return !!checkDateVoid(YearCalendar.getEndYear(Y) + 1, 1, 1, min, max)[0];
    }

    _getItems() {
        const { year } = this.options;
        const years = this._yearCreator(year || this.currentYear);

        // 纵向排列年
        const len = years.length,
            tyears = makeArray(len, "");
        const mapArr = [0, 6, 1, 7, 2, 8, 3, 9, 4, 10, 5, 11];
        each(years, (i, y) => {
            tyears[i] = years[mapArr[i]];
        });

        const items = [];
        items.push(tyears.slice(0, 2));
        items.push(tyears.slice(2, 4));
        items.push(tyears.slice(4, 6));
        items.push(tyears.slice(6, 8));
        items.push(tyears.slice(8, 10));
        items.push(tyears.slice(10, 12));

        return map(items, (i, item) =>
            map(item, (j, td) =>
                extend(td, {
                    type: TextItem.xtype,
                    cls: "bi-list-item-select bi-border-radius",
                    textAlign: "center",
                    whiteSpace: "normal",
                    once: false,
                    forceSelected: true,
                    height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
                    width: 45,
                    value: td.text,
                    disabled: td.disabled,
                })
            )
        );
    }

    _populate() {
        this.years.populate(this._getItems());
    }

    setMinDate(minDate) {
        const o = this.options;
        if (isNotEmptyString(o.min)) {
            o.min = minDate;
            this._populate();
        }
    }

    setMaxDate(maxDate) {
        const o = this.options;
        if (isNotEmptyString(this.options.max)) {
            o.max = maxDate;
            this._populate();
        }
    }

    setValue(val) {
        this.years.setValue([val]);
    }

    getValue() {
        return this.years.getValue()[0];
    }
}
