import {
    Layout,
    AbsoluteLayout,
    VerticalAdaptLayout,
    shortcut,
    Widget,
    extend,
    createWidgets,
    createItems,
    createWidget,
    each,
    isEmptyString,
    isNumeric,
    isNull,
    DOM,
    i18nText
} from "@/core";
import { Label, IconButton } from "@/base";
import { SmallTextEditor } from "@/widget/editor/editor.text.small.js";
import { TextEditor } from "@/widget/editor/editor.text";

const RGB_WIDTH = 32;

/**
 * 简单选色控件
 *
 * Created by GUY on 2015/11/16.
 * @class ColorPickerEditor
 * @extends Widget
 */
@shortcut()
export class ColorPickerEditor extends Widget {
    static xtype = "bi.color_picker_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-color-picker-editor",
            // width: 200,
            height: 30,
        });
    }

    _init() {
        super._init(...arguments);
        this.storeValue = {};
        this.colorShow = createWidget({
            type: Layout.xtype,
            cls: "color-picker-editor-display bi-card bi-border",
            height: 16,
            width: 16,
        });
        const RGB = createWidgets(
            createItems([{ text: "R" }, { text: "G" }, { text: "B" }], {
                type: Label.xtype,
                cls: "color-picker-editor-label",
                width: 20,
                height: 20,
            })
        );

        const checker = v => isNumeric(v) && (v | 0) >= 0 && (v | 0) <= 255;
        const Ws = createWidgets([{}, {}, {}], {
            type: SmallTextEditor.xtype,
            cls: "color-picker-editor-input bi-border-radius",
            validationChecker: checker,
            errorText: i18nText("BI-Color_Picker_Error_Text"),
            allowBlank: true,
            value: 255,
            width: RGB_WIDTH,
            height: 20,
        });
        each(Ws, (i, w) => {
            w.on(TextEditor.EVENT_CHANGE, () => {
                this._checkEditors();
                if (checker(this.storeValue.r) && checker(this.storeValue.g) && checker(this.storeValue.b)) {
                    this.colorShow.element.css("background-color", this.getValue());
                    this.fireEvent(ColorPickerEditor.EVENT_CHANGE);
                }
            });
        });
        this.R = Ws[0];
        this.G = Ws[1];
        this.B = Ws[2];

        this.none = createWidget({
            type: IconButton.xtype,
            cls: "auto-color-icon",
            width: 16,
            height: 16,
            iconWidth: 16,
            iconHeight: 16,
            title: i18nText("BI-Basic_Auto"),
        });
        this.none.on(IconButton.EVENT_CHANGE, () => {
            const value = this.getValue();
            this.setValue("");
            value !== "" && this.fireEvent(ColorPickerEditor.EVENT_CHANGE);
        });

        this.transparent = createWidget({
            type: IconButton.xtype,
            cls: "trans-color-icon",
            width: 16,
            height: 16,
            iconWidth: 16,
            iconHeight: 16,
            title: i18nText("BI-Transparent_Color"),
        });
        this.transparent.on(IconButton.EVENT_CHANGE, () => {
            const value = this.getValue();
            this.setValue("transparent");
            value !== "transparent" && this.fireEvent(ColorPickerEditor.EVENT_CHANGE);
        });

        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: {
                        type: VerticalAdaptLayout.xtype,
                        items: [
                            {
                                el: this.colorShow,
                                width: 16,
                            },
                            {
                                el: RGB[0],
                                width: 20,
                            },
                            {
                                el: this.R,
                                width: RGB_WIDTH,
                            },
                            {
                                el: RGB[1],
                                width: 20,
                            },
                            {
                                el: this.G,
                                width: RGB_WIDTH,
                            },
                            {
                                el: RGB[2],
                                width: 20,
                            },
                            {
                                el: this.B,
                                width: RGB_WIDTH,
                            },
                            {
                                el: this.transparent,
                                width: 16,
                                lgap: 5,
                            },
                            {
                                el: this.none,
                                width: 16,
                                lgap: 5,
                            }
                        ],
                    },
                    left: 10,
                    right: 10,
                    top: 0,
                    bottom: 0,
                }
            ],
        });
    }

    _checkEditors() {
        if (isEmptyString(this.R.getValue())) {
            this.R.setValue(0);
        }
        if (isEmptyString(this.G.getValue())) {
            this.G.setValue(0);
        }
        if (isEmptyString(this.B.getValue())) {
            this.B.setValue(0);
        }
        this.storeValue = {
            r: this.R.getValue() || 0,
            g: this.G.getValue() || 0,
            b: this.B.getValue() || 0,
        };
    }

    _isEmptyRGB() {
        return isEmptyString(this.storeValue.r) && isEmptyString(this.storeValue.g) && isEmptyString(this.storeValue.b);
    }

    _showPreColor(color) {
        if (color === "") {
            this.colorShow.element
                .css("background-color", "")
                .removeClass("trans-color-background")
                .addClass("auto-color-normal-background");
        } else if (color === "transparent") {
            this.colorShow.element
                .css("background-color", "")
                .removeClass("auto-color-normal-background")
                .addClass("trans-color-background");
        } else {
            this.colorShow.element
                .css({ "background-color": color })
                .removeClass("auto-color-normal-background")
                .removeClass("trans-color-background");
        }
    }

    _setEnable(enable) {
        super._setEnable(...arguments);
        if (enable === true) {
            this.element.removeClass("base-disabled disabled");
        } else if (enable === false) {
            this.element.addClass("base-disabled disabled");
        }
    }

    setValue(color) {
        if (color === "transparent") {
            this.transparent.setSelected(true);
            this.none.setSelected(false);
            this._showPreColor("transparent");
            this.R.setValue("");
            this.G.setValue("");
            this.B.setValue("");
            this.storeValue = {
                r: "",
                g: "",
                b: "",
            };

            return;
        }
        if (!color) {
            color = "";
            this.none.setSelected(true);
        } else {
            this.none.setSelected(false);
        }
        this.transparent.setSelected(false);
        this._showPreColor(color);
        const json = DOM.rgb2json(DOM.hex2rgb(color));
        this.storeValue = {
            r: isNull(json.r) ? "" : json.r,
            g: isNull(json.g) ? "" : json.g,
            b: isNull(json.b) ? "" : json.b,
        };
        this.R.setValue(this.storeValue.r);
        this.G.setValue(this.storeValue.g);
        this.B.setValue(this.storeValue.b);
    }

    getValue() {
        if (this._isEmptyRGB() && this.transparent.isSelected()) {
            return "transparent";
        }

        return DOM.rgb2hex(
            DOM.json2rgb({
                r: this.storeValue.r,
                g: this.storeValue.g,
                b: this.storeValue.b,
            })
        );
    }
}
