import { Editor, IconButton } from "@/base";
import {
    HTapeLayout,
    shortcut,
    Widget,
    extend,
    emptyFn,
    isKey,
    isFunction,
    createWidget,
    Controller,
    Events
} from "@/core";

/**
 * 有清除按钮的文本框
 * Created by GUY on 2015/9/29.
 * @class ClearEditor
 * @extends Widget
 */
@shortcut()
export class ClearEditor extends Widget {
    static xtype = "bi.clear_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_CLICK = "EVENT_CLICK";
    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_SPACE = "EVENT_SPACE";
    static EVENT_BACKSPACE = "EVENT_BACKSPACE";
    static EVENT_CLEAR = "EVENT_CLEAR";
    static EVENT_START = "EVENT_START";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE_CONFIRM = "EVENT_CHANGE_CONFIRM";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_ENTER = "EVENT_ENTER";
    static EVENT_RESTRICT = "EVENT_RESTRICT";
    static EVENT_REMOVE = "EVENT_REMOVE";
    static EVENT_EMPTY = "EVENT_EMPTY";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-clear-editor",
            height: 24,
            errorText: "",
            watermark: "",
            validationChecker: emptyFn,
            quitChecker: emptyFn,
        });
    }

    _init() {
        const o = this.options;
        o.value = isFunction(o.value)
            ? this.__watch(o.value, (context, newValue) => {
                this.setValue(newValue);
            })
            : o.value;
        super._init(...arguments);
        this.editor = createWidget({
            type: Editor.xtype,
            simple: o.simple,
            height: o.height,
            watermark: o.watermark,
            allowBlank: true,
            errorText: o.errorText,
            validationChecker: o.validationChecker,
            quitChecker: o.quitChecker,
            value: o.value,
            autoTrim: o.autoTrim,
        });
        this.clear = createWidget({
            type: IconButton.xtype,
            stopEvent: true,
            invisible: !isKey(o.value),
            cls: "search-close-h-font",
        });
        this.clear.on(IconButton.EVENT_CHANGE, () => {
            this.setValue("");
            this.fireEvent(Controller.EVENT_CHANGE, Events.STOPEDIT);
            this.fireEvent(ClearEditor.EVENT_CLEAR);
        });
        createWidget({
            element: this,
            type: HTapeLayout.xtype,
            items: [
                {
                    el: this.editor,
                },
                {
                    el: this.clear,
                    width: 24,
                }
            ],
        });
        this.editor.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });

        this.editor.on(Editor.EVENT_FOCUS, () => {
            this.fireEvent(ClearEditor.EVENT_FOCUS);
        });
        this.editor.on(Editor.EVENT_BLUR, () => {
            this.fireEvent(ClearEditor.EVENT_BLUR);
        });
        this.editor.on(Editor.EVENT_CLICK, () => {
            this.fireEvent(ClearEditor.EVENT_CLICK);
        });
        this.editor.on(Editor.EVENT_CHANGE, () => {
            this._checkClear();
            this.fireEvent(ClearEditor.EVENT_CHANGE);
        });
        this.editor.on(Editor.EVENT_KEY_DOWN, v => {
            this.fireEvent(ClearEditor.EVENT_KEY_DOWN, v);
        });
        this.editor.on(Editor.EVENT_SPACE, () => {
            this.fireEvent(ClearEditor.EVENT_SPACE);
        });
        this.editor.on(Editor.EVENT_BACKSPACE, () => {
            this.fireEvent(ClearEditor.EVENT_BACKSPACE);
        });

        this.editor.on(Editor.EVENT_VALID, () => {
            this.fireEvent(ClearEditor.EVENT_VALID);
        });
        this.editor.on(Editor.EVENT_ERROR, () => {
            this.fireEvent(ClearEditor.EVENT_ERROR);
        });
        this.editor.on(Editor.EVENT_ENTER, () => {
            this.fireEvent(ClearEditor.EVENT_ENTER);
        });
        this.editor.on(Editor.EVENT_RESTRICT, () => {
            this.fireEvent(ClearEditor.EVENT_RESTRICT);
        });
        this.editor.on(Editor.EVENT_EMPTY, () => {
            this._checkClear();
            this.fireEvent(ClearEditor.EVENT_EMPTY);
        });
        this.editor.on(Editor.EVENT_REMOVE, () => {
            this.fireEvent(ClearEditor.EVENT_REMOVE);
        });
        this.editor.on(Editor.EVENT_CONFIRM, () => {
            this.fireEvent(ClearEditor.EVENT_CONFIRM);
        });
        this.editor.on(Editor.EVENT_CHANGE_CONFIRM, () => {
            this.fireEvent(ClearEditor.EVENT_CHANGE_CONFIRM);
        });
        this.editor.on(Editor.EVENT_START, () => {
            this.fireEvent(ClearEditor.EVENT_START);
        });
        this.editor.on(Editor.EVENT_PAUSE, () => {
            this.fireEvent(ClearEditor.EVENT_PAUSE);
        });
        this.editor.on(Editor.EVENT_STOP, () => {
            this.fireEvent(ClearEditor.EVENT_STOP);
        });
    }

    _checkClear() {
        if (!this.getValue()) {
            this.clear.invisible();
        } else {
            this.clear.visible();
        }
    }

    setWaterMark(v) {
        this.options.watermark = v;
        this.editor.setWaterMark(v);
    }

    focus() {
        this.editor.focus();
    }

    blur() {
        this.editor.blur();
    }

    getValue() {
        if (this.isValid()) {
            return this.editor.getValue();
        }
    }

    setValue(v) {
        this.editor.setValue(v);
        if (isKey(v)) {
            this.clear.visible();
        }
    }

    isValid() {
        return this.editor.isValid();
    }
}
