import { Editor, TextButton } from "@/base";
import {
    AbsoluteLayout,
    shortcut,
    Widget,
    emptyFn,
    isKey,
    isFunction,
    createWidget,
    nextTick,
    Controller
} from "@/core";

/**
 * dailer
 * 有默认提示文字的输入框
 * @class DefaultTextEditor
 * @extends Widget
 */
@shortcut()
export class DefaultTextEditor extends Widget {
    static xtype = "bi.default_text_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_CLICK = "EVENT_CLICK";
    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_CLICK_LABEL = "EVENT_CLICK_LABEL";
    static EVENT_START = "EVENT_START";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE_CONFIRM = "EVENT_CHANGE_CONFIRM";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_ENTER = "EVENT_ENTER";
    static EVENT_RESTRICT = "EVENT_RESTRICT";
    static EVENT_SPACE = "EVENT_SPACE";
    static EVENT_EMPTY = "EVENT_EMPTY";

    props() {
        return {
            baseCls: "bi-default-text-editor",
            hgap: 4,
            vgap: 2,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            validationChecker: emptyFn,
            quitChecker: emptyFn,
            allowBlank: true,
            watermark: "",
            errorText: "",
            height: 24,
            defaultText: "", // 默认显示值，默认显示值与显示值的区别是默认显示值标记灰色
            text: "", // 显示值
            el: {},
        };
    }

    render() {
        const o = this.options;
        this.editor = createWidget(o.el, {
            type: Editor.xtype,
            simple: o.simple,
            height: o.height,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            value: o.value,
            validationChecker: o.validationChecker,
            quitChecker: o.quitChecker,
            allowBlank: o.allowBlank,
            watermark: o.watermark,
            errorText: o.errorText,
            invisible: true,
            autoTrim: o.autoTrim,
        });

        const showText = isFunction(o.text) ? o.text() : o.text;

        this.text = createWidget({
            type: TextButton.xtype,
            cls: isKey(showText) ? "tip-text-style" : "bi-water-mark tip-text-style",
            textAlign: "left",
            height: o.height,
            text: showText || o.defaultText,
            hgap: o.hgap + 2,
            handler: () => {
                this._showInput();
                this.editor.focus();
                this.editor.setValue("");
            },
            title: o.title,
            warningTitle: o.warningTitle,
            tipType: o.tipType,
        });
        this.text.on(TextButton.EVENT_CHANGE, () => {
            nextTick(() => {
                this.fireEvent(DefaultTextEditor.EVENT_CLICK_LABEL);
            });
        });

        this.editor.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        this.editor.on(Editor.EVENT_FOCUS, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_FOCUS, ...args);
        });
        this.editor.on(Editor.EVENT_BLUR, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_BLUR, ...args);
        });
        this.editor.on(Editor.EVENT_CLICK, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_CLICK, ...args);
        });
        this.editor.on(Editor.EVENT_CHANGE, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_CHANGE, ...args);
        });
        this.editor.on(Editor.EVENT_KEY_DOWN, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_KEY_DOWN, ...args);
        });

        this.editor.on(Editor.EVENT_VALID, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_VALID, ...args);
        });
        this.editor.on(Editor.EVENT_CONFIRM, (...args) => {
            this._showHint();
            this.fireEvent(DefaultTextEditor.EVENT_CONFIRM, ...args);
        });
        this.editor.on(Editor.EVENT_CHANGE_CONFIRM, (...args) => {
            this._showHint();
            this.fireEvent(DefaultTextEditor.EVENT_CHANGE_CONFIRM, ...args);
        });
        this.editor.on(Editor.EVENT_START, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_START, ...args);
        });
        this.editor.on(Editor.EVENT_PAUSE, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_PAUSE, ...args);
        });
        this.editor.on(Editor.EVENT_STOP, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_STOP, ...args);
        });
        this.editor.on(Editor.EVENT_SPACE, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_SPACE, ...args);
        });
        this.editor.on(Editor.EVENT_ERROR, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_ERROR, ...args);
        });
        this.editor.on(Editor.EVENT_ENTER, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_ENTER, ...args);
        });
        this.editor.on(Editor.EVENT_RESTRICT, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_RESTRICT, ...args);
        });
        this.editor.on(Editor.EVENT_EMPTY, (...args) => {
            this.fireEvent(DefaultTextEditor.EVENT_EMPTY, ...args);
        });

        return {
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: this.editor,
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                },
                {
                    el: this.text,
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                }
            ],
        };
    }

    setWaterMark(v) {
        this.options.watermark = v;
        this.editor.setWaterMark(v);
    }

    setTitle(title) {
        this.text.setTitle(title);
    }

    setWarningTitle(title) {
        this.text.setWarningTitle(title);
    }

    doRedMark() {
        if (this.editor.getValue() === "" && isKey(this.options.watermark)) {
            return;
        }
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doHighLight() {
        if (this.editor.getValue() === "" && isKey(this.options.watermark)) {
            return;
        }
        this.text.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }

    focus() {
        if (this.options.disabled === false) {
            this._showInput();
            this.editor.focus();
        }
    }

    blur() {
        this.editor.blur();
        this._showHint();
    }

    _showInput() {
        this.editor.visible();
        this.text.invisible();
    }

    _showHint() {
        this.editor.invisible();
        this.text.visible();
    }

    _setText(v) {
        this.text.setText(v);
        this.text.setTitle(v);
    }

    isValid() {
        return this.editor.isValid();
    }

    setErrorText(text) {
        this.editor.setErrorText(text);
    }

    getErrorText() {
        return this.editor.getErrorText();
    }

    isEditing() {
        return this.editor.isEditing();
    }

    getLastValidValue() {
        return this.editor.getLastValidValue();
    }

    getLastChangedValue() {
        return this.editor.getLastChangedValue();
    }

    setValue(k) {
        this.editor.setValue(k);
    }

    getValue() {
        return this.editor.getValue();
    }

    getState() {
        return this.text.getValue();
    }

    setState(v) {
        const o = this.options;
        if (isKey(v)) {
            this.text.setText(v);
            this.text.element.removeClass("bi-water-mark");

            return;
        }
        this.text.setText(o.defaultText);
        this.text.element.addClass("bi-water-mark");
    }

    setTipType(v) {
        this.text.options.tipType = v;
    }

    getText() {
        return this.text.getText();
    }
}
