import { Editor, TextButton } from "@/base";
import {
    AbsoluteLayout,
    shortcut,
    Widget,
    extend,
    emptyFn,
    isFunction,
    createWidget,
    nextTick,
    isKey,
    bind,
    Controller
} from "@/core";

/**
 * 带标记的文本框
 * Created by GUY on 2015/8/28.
 * @class SignEditor
 * @extends Widget
 */
@shortcut()
export class SignEditor extends Widget {
    static xtype = "bi.sign_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_CLICK = "EVENT_CLICK";
    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_QUICK_DOWN = "EVENT_QUICK_DOWN";
    static EVENT_CLICK_LABEL = "EVENT_CLICK_LABEL";
    static EVENT_START = "EVENT_START";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE_CONFIRM = "EVENT_CHANGE_CONFIRM";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_ENTER = "EVENT_ENTER";
    static EVENT_RESTRICT = "EVENT_RESTRICT";
    static EVENT_SPACE = "EVENT_SPACE";
    static EVENT_EMPTY = "EVENT_EMPTY";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-sign-editor`,
            hgap: 4,
            vgap: 2,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            validationChecker: emptyFn,
            quitChecker: emptyFn,
            allowBlank: true,
            watermark: "",
            errorText: "",
            textAlign: "left",
            height: 24,
        });
    }

    _init() {
        const o = this.options;
        o.value = isFunction(o.value)
            ? this.__watch(o.value, (context, newValue) => {
                this.setValue(newValue);
            })
            : o.value;
        super._init(...arguments);
        this.editor = createWidget({
            type: Editor.xtype,
            simple: o.simple,
            height: o.height,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            value: o.value,
            validationChecker: o.validationChecker,
            quitChecker: o.quitChecker,
            allowBlank: o.allowBlank,
            watermark: o.watermark,
            errorText: o.errorText,
            autoTrim: o.autoTrim,
        });
        this.text = createWidget({
            type: TextButton.xtype,
            cls: "sign-editor-text",
            title: o.title,
            warningTitle: o.warningTitle,
            tipType: o.tipType,
            textAlign: o.textAlign,
            height: o.height,
            hgap: o.hgap + 2,
            handler: () => {
                this._showInput();
                this.editor.focus();
                this.editor.selectAll();
            },
        });
        this.text.on(TextButton.EVENT_CHANGE, () => {
            nextTick(() => {
                this.fireEvent(SignEditor.EVENT_CLICK_LABEL);
            });
        });
        this.editor.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        this.editor.on(Editor.EVENT_FOCUS, (...args) => {
            this.fireEvent(SignEditor.EVENT_FOCUS, ...args);
        });
        this.editor.on(Editor.EVENT_BLUR, (...args) => {
            this.fireEvent(SignEditor.EVENT_BLUR, ...args);
        });
        this.editor.on(Editor.EVENT_CLICK, (...args) => {
            this.fireEvent(SignEditor.EVENT_CLICK, ...args);
        });
        this.editor.on(Editor.EVENT_CHANGE, (...args) => {
            this.fireEvent(SignEditor.EVENT_CHANGE, ...args);
        });
        this.editor.on(Editor.EVENT_KEY_DOWN, (...args) => {
            this.fireEvent(SignEditor.EVENT_KEY_DOWN, ...args);
        });
        this.editor.on(Editor.EVENT_QUICK_DOWN, (...args) => {
            this.fireEvent(SignEditor.EVENT_QUICK_DOWN, ...args);
        });

        this.editor.on(Editor.EVENT_VALID, (...args) => {
            this.fireEvent(SignEditor.EVENT_VALID, ...args);
        });
        this.editor.on(Editor.EVENT_CONFIRM, (...args) => {
            this._showHint();
            this._checkText();
            this.fireEvent(SignEditor.EVENT_CONFIRM, ...args);
        });
        this.editor.on(Editor.EVENT_CHANGE_CONFIRM, (...args) => {
            this._showHint();
            this._checkText();
            this.fireEvent(SignEditor.EVENT_CHANGE_CONFIRM, ...args);
        });
        this.editor.on(Editor.EVENT_START, (...args) => {
            this.fireEvent(SignEditor.EVENT_START, ...args);
        });
        this.editor.on(Editor.EVENT_PAUSE, (...args) => {
            this.fireEvent(SignEditor.EVENT_PAUSE, ...args);
        });
        this.editor.on(Editor.EVENT_STOP, (...args) => {
            this.fireEvent(SignEditor.EVENT_STOP, ...args);
        });
        this.editor.on(Editor.EVENT_SPACE, (...args) => {
            this.fireEvent(SignEditor.EVENT_SPACE, ...args);
        });
        this.editor.on(Editor.EVENT_ERROR, (...args) => {
            this._checkText();
            this.fireEvent(SignEditor.EVENT_ERROR, ...args);
        });
        this.editor.on(Editor.EVENT_ENTER, (...args) => {
            this.fireEvent(SignEditor.EVENT_ENTER, ...args);
        });
        this.editor.on(Editor.EVENT_RESTRICT, (...args) => {
            this.fireEvent(SignEditor.EVENT_RESTRICT, ...args);
        });
        this.editor.on(Editor.EVENT_EMPTY, (...args) => {
            this.fireEvent(SignEditor.EVENT_EMPTY, ...args);
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.text,
                    inset: 0,
                },
                {
                    el: this.editor,
                    inset: 0,
                }
            ],
        });
        this._showHint();
        this._checkText();
    }

    _checkText() {
        const o = this.options;
        nextTick(
            bind(function () {
                if (this.editor.getValue() === "") {
                    this.text.setValue(o.watermark || "");
                    this.text.element.addClass("bi-water-mark");
                } else {
                    this.text.setValue(this.editor.getValue());
                    this.text.element.removeClass("bi-water-mark");
                    isKey(o.keyword) && this.text.doRedMark(o.keyword);
                }
            }, this)
        );
    }

    _showInput() {
        this.editor.visible();
        this.text.invisible();
    }

    _showHint() {
        this.editor.invisible();
        this.text.visible();
    }

    setTitle(title) {
        this.text.setTitle(title);
    }

    setTipType(v) {
        this.text.setTipType(v);
    }

    setWarningTitle(title) {
        this.text.setWarningTitle(title);
    }

    setWaterMark(v) {
        this.options.watermark = v;
        this._checkText();
        this.editor.setWaterMark(v);
    }

    focus() {
        this._showInput();
        this.editor.focus();
    }

    blur() {
        this.editor.blur();
        this._showHint();
        this._checkText();
    }

    doRedMark() {
        if (this.editor.getValue() === "" && isKey(this.options.watermark)) {
            return;
        }
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doHighLight() {
        if (this.editor.getValue() === "" && isKey(this.options.watermark)) {
            return;
        }
        this.text.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }

    isValid() {
        return this.editor.isValid();
    }

    setErrorText(text) {
        this.editor.setErrorText(text);
    }

    getErrorText() {
        return this.editor.getErrorText();
    }

    isEditing() {
        return this.editor.isEditing();
    }

    getLastValidValue() {
        return this.editor.getLastValidValue();
    }

    getLastChangedValue() {
        return this.editor.getLastChangedValue();
    }

    setValue(k) {
        this.editor.setValue(k);
        this._checkText();
    }

    getValue() {
        return this.editor.getValue();
    }

    getState() {
        return this.text.getValue();
    }

    setState(v) {
        this._showHint();
        this.text.setValue(v);
    }
}
