import { Editor, TextButton } from "@/base";
import {
    AbsoluteLayout,
    shortcut,
    Widget,
    extend,
    emptyFn,
    i18nText,
    isArray,
    createWidget,
    nextTick,
    Controller,
    isNotNull,
    isString,
    isKey,
    isFunction,
    isNumber,
    isEmpty,
    Selection
} from "@/core";

/**
 * guy
 * 记录状态的输入框
 * @class StateEditor
 * @extends Single
 */
@shortcut()
export class StateEditor extends Widget {
    static xtype = "bi.state_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_CLICK = "EVENT_CLICK";
    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_CLICK_LABEL = "EVENT_CLICK_LABEL";
    static EVENT_START = "EVENT_START";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE_CONFIRM = "EVENT_CHANGE_CONFIRM";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_ENTER = "EVENT_ENTER";
    static EVENT_RESTRICT = "EVENT_RESTRICT";
    static EVENT_SPACE = "EVENT_SPACE";
    static EVENT_EMPTY = "EVENT_EMPTY";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-state-editor`,
            hgap: 4,
            vgap: 2,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            validationChecker: emptyFn,
            quitChecker: emptyFn,
            allowBlank: true,
            watermark: "",
            errorText: "",
            height: 24,
            defaultText: i18nText("BI-Basic_Unrestricted"), // 默认显示值，默认显示值与显示值的区别是默认显示值标记灰色
            text: "", // 显示值
            el: {},
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.editor = createWidget(o.el, {
            type: Editor.xtype,
            simple: o.simple,
            height: o.height,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            value: o.value,
            validationChecker: o.validationChecker,
            quitChecker: o.quitChecker,
            allowBlank: o.allowBlank,
            watermark: o.watermark,
            errorText: o.errorText,
            autoTrim: o.autoTrim,
        });
        this.text = createWidget({
            type: TextButton.xtype,
            cls: "bi-water-mark tip-text-style",
            textAlign: "left",
            height: o.height,
            text: o.text,
            hgap: o.hgap + 2,
            handler: () => {
                this._showInput();
                this.editor.focus();
                this.editor.setValue("");
            },
            title: isNotNull(o.tipText)
                ? o.tipText
                : () => {
                    let title = "";
                    if (isString(this.stateValue)) {
                        title = this.stateValue;
                    }
                    if (isArray(this.stateValue) && this.stateValue.length === 1) {
                        title = this.stateValue[0];
                    }

                    return title;
                },
            warningTitle: o.warningTitle,
            tipType: o.tipType,
        });
        this.text.on(TextButton.EVENT_CHANGE, () => {
            nextTick(() => {
                this.fireEvent(StateEditor.EVENT_CLICK_LABEL);
            });
        });
        this.editor.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        this.editor.on(Editor.EVENT_FOCUS, (...args) => {
            this.fireEvent(StateEditor.EVENT_FOCUS, ...args);
        });
        this.editor.on(Editor.EVENT_BLUR, (...args) => {
            this.fireEvent(StateEditor.EVENT_BLUR, ...args);
        });
        this.editor.on(Editor.EVENT_CLICK, (...args) => {
            this.fireEvent(StateEditor.EVENT_CLICK, ...args);
        });
        this.editor.on(Editor.EVENT_CHANGE, (...args) => {
            this.fireEvent(StateEditor.EVENT_CHANGE, ...args);
        });
        this.editor.on(Editor.EVENT_KEY_DOWN, (...args) => {
            this.fireEvent(StateEditor.EVENT_KEY_DOWN, ...args);
        });

        this.editor.on(Editor.EVENT_VALID, (...args) => {
            this.fireEvent(StateEditor.EVENT_VALID, ...args);
        });
        this.editor.on(Editor.EVENT_CONFIRM, (...args) => {
            this._showHint();
            this.fireEvent(StateEditor.EVENT_CONFIRM, ...args);
        });
        this.editor.on(Editor.EVENT_CHANGE_CONFIRM, (...args) => {
            this._showHint();
            this.fireEvent(StateEditor.EVENT_CHANGE_CONFIRM, ...args);
        });
        this.editor.on(Editor.EVENT_START, (...args) => {
            this.fireEvent(StateEditor.EVENT_START, ...args);
        });
        this.editor.on(Editor.EVENT_PAUSE, (...args) => {
            this.fireEvent(StateEditor.EVENT_PAUSE, ...args);
        });
        this.editor.on(Editor.EVENT_STOP, (...args) => {
            this.fireEvent(StateEditor.EVENT_STOP, ...args);
        });
        this.editor.on(Editor.EVENT_SPACE, (...args) => {
            this.fireEvent(StateEditor.EVENT_SPACE, ...args);
        });
        this.editor.on(Editor.EVENT_ERROR, (...args) => {
            this.fireEvent(StateEditor.EVENT_ERROR, ...args);
        });
        this.editor.on(Editor.EVENT_ENTER, (...args) => {
            this.fireEvent(StateEditor.EVENT_ENTER, ...args);
        });
        this.editor.on(Editor.EVENT_RESTRICT, (...args) => {
            this.fireEvent(StateEditor.EVENT_RESTRICT, ...args);
        });
        this.editor.on(Editor.EVENT_EMPTY, (...args) => {
            this.fireEvent(StateEditor.EVENT_EMPTY, ...args);
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.text,
                    inset: 0,
                },
                {
                    el: this.editor,
                    inset: 0,
                }
            ],
        });
        this._showHint();
        if (isNotNull(o.text)) {
            this.setState(o.text);
        }
    }

    setWaterMark(v) {
        this.options.watermark = v;
        this.editor.setWaterMark(v);
    }

    doRedMark() {
        if (this.editor.getValue() === "" && isKey(this.options.watermark)) {
            return;
        }
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doHighLight() {
        if (this.editor.getValue() === "" && isKey(this.options.watermark)) {
            return;
        }
        this.text.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }

    focus() {
        if (this.options.disabled === false) {
            this._showInput();
            this.editor.focus();
        }
    }

    blur() {
        this.editor.blur();
        this._showHint();
    }

    _showInput() {
        this.editor.visible();
        this.text.invisible();
    }

    _showHint() {
        this.editor.invisible();
        this.text.visible();
    }

    _setText(v) {
        this.text.setText(v);
        this.text.setTitle(v);
    }

    isValid() {
        return this.editor.isValid();
    }

    setErrorText(text) {
        this.editor.setErrorText(text);
    }

    getErrorText() {
        return this.editor.getErrorText();
    }

    isEditing() {
        return this.editor.isEditing();
    }

    getLastValidValue() {
        return this.editor.getLastValidValue();
    }

    getLastChangedValue() {
        return this.editor.getLastChangedValue();
    }

    setValue(k) {
        this.editor.setValue(k);
    }

    getValue() {
        return this.editor.getValue();
    }

    getState() {
        return this.editor.getValue().match(/[^\s]+/g);
    }

    setState(v) {
        const o = this.options;
        const defaultText = isFunction(o.defaultText) ? o.defaultText() : o.defaultText;
        super.setValue(...arguments);
        this.stateValue = v;
        if (isNumber(v)) {
            if (v === Selection.All) {
                this._setText(i18nText("BI-Select_All"));
                this.text.element.removeClass("bi-water-mark");
            } else if (v === Selection.Multi) {
                this._setText(i18nText("BI-Select_Part"));
                this.text.element.removeClass("bi-water-mark");
            } else {
                this._setText(isKey(defaultText) ? defaultText : o.text);
                isKey(defaultText)
                    ? this.text.element.addClass("bi-water-mark")
                    : this.text.element.removeClass("bi-water-mark");
            }

            return;
        }
        if (isString(v)) {
            this._setText(v);
            // 配置了defaultText才判断标灰，其他情况不标灰
            isKey(defaultText) && defaultText === v
                ? this.text.element.addClass("bi-water-mark")
                : this.text.element.removeClass("bi-water-mark");

            return;
        }
        if (isArray(v)) {
            if (isEmpty(v)) {
                this._setText(isKey(defaultText) ? defaultText : o.text);
                isKey(defaultText)
                    ? this.text.element.addClass("bi-water-mark")
                    : this.text.element.removeClass("bi-water-mark");
            } else if (v.length === 1) {
                this._setText(v[0]);
                this.text.element.removeClass("bi-water-mark");
            } else {
                this._setText(i18nText("BI-Select_Part"));
                this.text.element.removeClass("bi-water-mark");
            }
        }
    }

    setTipType(v) {
        this.text.options.tipType = v;
    }

    getText() {
        return this.text.getText();
    }
}
