import { Pager, IconButton, Label } from "@/base";
import {
    HorizontalLayout,
    VerticalAdaptLayout,
    FloatLeftLayout,
    shortcut,
    Widget,
    extend,
    isPositiveInteger,
    createWidget,
    parseInt,
    HorizontalAlign,
    isNotEmptyObject,
    i18nText
} from "@/core";
import { SmallTextEditor } from "@/widget/editor/editor.text.small";
import { TextEditor } from "@/widget/editor/editor.text";

/**
 * 有总页数和总行数的分页控件
 * Created by Young's on 2016/10/13.
 */

//

@shortcut()
export class AllCountPager extends Widget {
    static xtype = "bi.all_count_pager";
    static EVENT_CHANGE = "EVENT_CHANGE";
    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-all-count-pager",
            pagerDirection: "vertical", // 翻页按钮方向，可选值：vertical/horizontal
            height: 24,
            pages: 1, // 必选项
            curr: 1, // 初始化当前页， pages为数字时可用，
            count: 1, // 总行数
            rowInfoObject: null,
            showRowCount: true,
            showRowInfo: true,
        });
    }
    _init() {
        super._init(...arguments);
        const { pages, curr, hasPrev, hasNext, firstPage, lastPage, height, showRowCount } = this.options,
            pagerIconCls = this._getPagerIconCls();
        this.editor = createWidget({
            type: SmallTextEditor.xtype,
            cls: "pager-editor bi-border-radius",
            validationChecker(v) {
                return (pages === 0 && v === "0") || isPositiveInteger(v);
            },
            hgap: 4,
            vgap: 0,
            value: curr,
            errorText: i18nText("BI-Please_Input_Positive_Integer"),
            width: 40,
            height: 24,
            invisible: pages <= 1,
        });

        this.pager = createWidget({
            type: Pager.xtype,
            width: 58,
            layouts: [
                {
                    type: HorizontalLayout.xtype,
                    lgap: 5,
                }
            ],

            dynamicShow: false,
            pages,
            curr,
            groups: 0,

            first: false,
            last: false,
            prev: {
                type: IconButton.xtype,
                value: "prev",
                title: i18nText("BI-Previous_Page"),
                warningTitle: i18nText("BI-Current_Is_First_Page"),
                height: 22,
                width: 22,
                cls: `bi-border bi-border-radius all-pager-prev bi-list-item-select2 ${pagerIconCls.preCls}`,
            },
            next: {
                type: IconButton.xtype,
                value: "next",
                title: i18nText("BI-Next_Page"),
                warningTitle: i18nText("BI-Current_Is_Last_Page"),
                height: 22,
                width: 22,
                cls: `bi-border bi-border-radius all-pager-next bi-list-item-select2 ${pagerIconCls.nextCls}`,
            },

            hasPrev,
            hasNext,
            firstPage,
            lastPage,
            invisible: pages <= 1,
        });

        this.editor.on(TextEditor.EVENT_CONFIRM, () => {
            this.pager.setValue(parseInt(this.editor.getValue()));
            this.fireEvent(AllCountPager.EVENT_CHANGE);
        });
        this.pager.on(Pager.EVENT_CHANGE, () => {
            this.fireEvent(AllCountPager.EVENT_CHANGE);
        });
        this.pager.on(Pager.EVENT_AFTER_POPULATE, () => {
            this.editor.setValue(this.pager.getCurrentPage());
        });

        this.allPages = createWidget({
            type: Label.xtype,
            title: pages,
            height,
            text: `/${pages}`,
            lgap: 5,
            invisible: pages <= 1,
        });

        createWidget(
            showRowCount
                ? {
                    type: VerticalAdaptLayout.xtype,
                    element: this,
                    scrollx: false,
                    columnSize: ["fill", ""],
                    horizontalAlign: HorizontalAlign.Right,
                    items: [this._getRowCountObject(), this.editor, this.allPages, this.pager],
                }
                : {
                    type: VerticalAdaptLayout.xtype,
                    element: this,
                    items: [this.editor, this.allPages, this.pager],
                }
        );
    }

    _getPagerIconCls() {
        const { pagerDirection } = this.options;
        switch (pagerDirection) {
            case "horizontal":
                return {
                    preCls: "row-pre-page-h-font ",
                    nextCls: "row-next-page-h-font ",
                };
            case "vertical":
            default:
                return {
                    preCls: "column-pre-page-h-font ",
                    nextCls: "column-next-page-h-font ",
                };
        }
    }

    _getRowCountObject() {
        const { height, count, rowInfoObject } = this.options;

        return {
            type: FloatLeftLayout.xtype,
            height,
            scrollable: false,
            ref: _ref => {
                this.rowCountObject = _ref;
            },
            items: [
                {
                    type: Label.xtype,
                    height,
                    text: i18nText("BI-Basic_Total"),
                    ref: _ref => {
                        this.prevText = _ref;
                    },
                },
                {
                    el: {
                        type: Label.xtype,
                        ref: _ref => {
                            this.rowCount = _ref;
                        },
                        cls: "row-count",
                        height,
                        text: count,
                        title: count,
                    },
                    hgap: 5,
                },
                {
                    type: Label.xtype,
                    height,
                    text: i18nText("BI-Tiao_Data"),
                    textAlign: "left",
                },
                isNotEmptyObject(rowInfoObject) ? rowInfoObject : null
            ],
        };
    }

    setAllPages(v) {
        this.allPages.setText(`/${v}`);
        this.allPages.setTitle(v);
        this.options.pages = v;
        this.pager.setAllPages(v);
        this.editor.setEnable(v >= 1);
        this.setPagerVisible(v > 1);
    }

    setShowRowInfo(b) {
        this.options.showRowInfo = b;
        this.rowCountObject.setVisible(b);
    }

    setValue(v) {
        this.pager.setValue(v);
    }

    setVPage(v) {
        this.pager.setValue(v);
    }

    setCount(count) {
        if (this.options.showRowCount) {
            this.rowCount.setText(count);
            this.rowCount.setTitle(count);
        }
    }

    setCountPrevText(text) {
        if (this.options.showRowCount) {
            this.prevText.setText(text);
        }
    }

    getCurrentPage() {
        return this.pager.getCurrentPage();
    }

    hasPrev() {
        return this.pager.hasPrev();
    }

    hasNext() {
        return this.pager.hasNext();
    }

    isShowPager() {
        return this.options.showRowInfo || this.options.pages > 1;
    }

    setPagerVisible(b) {
        this.editor.setVisible(b);
        this.allPages.setVisible(b);
        this.pager.setVisible(b);
    }

    populate() {
        this.pager.populate();
        this.setPagerVisible(this.options.pages > 1);
    }
}
