export class CellSizeAndPositionManager {

    constructor(cellCount, cellSizeGetter, estimatedCellSize) {
        this._cellSizeGetter = cellSizeGetter;
        this._cellCount = cellCount;
        this._estimatedCellSize = estimatedCellSize;
        this._cellSizeAndPositionData = {};
        this._lastMeasuredIndex = -1;
    }

    configure(cellCount, estimatedCellSize) {
        this._cellCount = cellCount;
        this._estimatedCellSize = estimatedCellSize;
    }

    getCellCount() {
        return this._cellCount;
    }

    getEstimatedCellSize() {
        return this._estimatedCellSize;
    }

    getLastMeasuredIndex() {
        return this._lastMeasuredIndex;
    }

    getSizeAndPositionOfCell(index) {
        if (index < 0 || index >= this._cellCount) {
            return;
        }
        if (index > this._lastMeasuredIndex) {
            const lastMeasuredCellSizeAndPosition = this.getSizeAndPositionOfLastMeasuredCell();
            let offset = lastMeasuredCellSizeAndPosition.offset + lastMeasuredCellSizeAndPosition.size;

            for (let i = this._lastMeasuredIndex + 1; i <= index; i++) {
                const size = this._cellSizeGetter(i);

                if (size == null || isNaN(size)) {
                    continue;
                }

                this._cellSizeAndPositionData[i] = {
                    offset: offset,
                    size: size,
                };

                offset += size;
            }

            this._lastMeasuredIndex = index;
        }
        return this._cellSizeAndPositionData[index];
    }

    getSizeAndPositionOfLastMeasuredCell() {
        return this._lastMeasuredIndex >= 0
            ? this._cellSizeAndPositionData[this._lastMeasuredIndex]
            : {
                offset: 0,
                size: 0,
            };
    }

    getTotalSize() {
        const lastMeasuredCellSizeAndPosition = this.getSizeAndPositionOfLastMeasuredCell();
        return lastMeasuredCellSizeAndPosition.offset + lastMeasuredCellSizeAndPosition.size + (this._cellCount - this._lastMeasuredIndex - 1) * this._estimatedCellSize;
    }

    getUpdatedOffsetForIndex(align, containerSize, currentOffset, targetIndex) {
        const datum = this.getSizeAndPositionOfCell(targetIndex);
        const maxOffset = datum.offset;
        const minOffset = maxOffset - containerSize + datum.size;

        let idealOffset;

        switch (align) {
            case "start":
                idealOffset = maxOffset;
                break;
            case "end":
                idealOffset = minOffset;
                break;
            case "center":
                idealOffset = maxOffset - ((containerSize - datum.size) / 2);
                break;
            default:
                idealOffset = Math.max(minOffset, Math.min(maxOffset, currentOffset));
                break;
        }

        const totalSize = this.getTotalSize();

        return Math.max(0, Math.min(totalSize - containerSize, idealOffset));
    }

    getVisibleCellRange(containerSize, offset) {
        const totalSize = this.getTotalSize();

        if (totalSize === 0) {
            return {};
        }

        const maxOffset = offset + containerSize;
        const start = this._findNearestCell(offset);

        const datum = this.getSizeAndPositionOfCell(start);
        offset = datum.offset + datum.size;

        let stop = start;

        while (offset < maxOffset && stop < this._cellCount - 1) {
            stop++;
            offset += this.getSizeAndPositionOfCell(stop).size;
        }

        return {
            start: start,
            stop: stop,
        };
    }

    resetCell(index) {
        this._lastMeasuredIndex = Math.min(this._lastMeasuredIndex, index - 1);
    }

    _binarySearch(high, low, offset) {
        let middle;
        let currentOffset;

        while (low <= high) {
            middle = low + Math.floor((high - low) / 2);
            currentOffset = this.getSizeAndPositionOfCell(middle).offset;

            if (currentOffset === offset) {
                return middle;
            } else if (currentOffset < offset) {
                low = middle + 1;
            } else if (currentOffset > offset) {
                high = middle - 1;
            }
        }

        if (low > 0) {
            return low - 1;
        }
    }

    _exponentialSearch(index, offset) {
        let interval = 1;

        while (index < this._cellCount && this.getSizeAndPositionOfCell(index).offset < offset) {
            index += interval;
            interval *= 2;
        }

        return this._binarySearch(Math.min(index, this._cellCount - 1), Math.floor(index / 2), offset);
    }

    _findNearestCell(offset) {
        if (isNaN(offset)) {
            return;
        }

        offset = Math.max(0, offset);

        const lastMeasuredCellSizeAndPosition = this.getSizeAndPositionOfLastMeasuredCell();
        const lastMeasuredIndex = Math.max(0, this._lastMeasuredIndex);

        if (lastMeasuredCellSizeAndPosition.offset >= offset) {
            return this._binarySearch(lastMeasuredIndex, 0, offset);
        }
        return this._exponentialSearch(lastMeasuredIndex, offset);

    }
}

export class ScalingCellSizeAndPositionManager {
    constructor(cellCount, cellSizeGetter, estimatedCellSize, maxScrollSize) {
        this._cellSizeAndPositionManager = new CellSizeAndPositionManager(cellCount, cellSizeGetter, estimatedCellSize);
        this._maxScrollSize = maxScrollSize || 10000000;
    }

    configure() {
        this._cellSizeAndPositionManager.configure.apply(this._cellSizeAndPositionManager, arguments);
    }

    getCellCount() {
        return this._cellSizeAndPositionManager.getCellCount();
    }

    getEstimatedCellSize() {
        return this._cellSizeAndPositionManager.getEstimatedCellSize();
    }

    getLastMeasuredIndex() {
        return this._cellSizeAndPositionManager.getLastMeasuredIndex();
    }

    getOffsetAdjustment(containerSize, offset) {
        const totalSize = this._cellSizeAndPositionManager.getTotalSize();
        const safeTotalSize = this.getTotalSize();
        const offsetPercentage = this._getOffsetPercentage(containerSize, offset, safeTotalSize);

        return Math.round(offsetPercentage * (safeTotalSize - totalSize));
    }

    getSizeAndPositionOfCell(index) {
        return this._cellSizeAndPositionManager.getSizeAndPositionOfCell(index);
    }

    getSizeAndPositionOfLastMeasuredCell() {
        return this._cellSizeAndPositionManager.getSizeAndPositionOfLastMeasuredCell();
    }

    getTotalSize() {
        return Math.min(this._maxScrollSize, this._cellSizeAndPositionManager.getTotalSize());
    }

    getUpdatedOffsetForIndex(align, containerSize, currentOffset, targetIndex) {
        currentOffset = this._safeOffsetToOffset(containerSize, currentOffset);

        const offset = this._cellSizeAndPositionManager.getUpdatedOffsetForIndex(align, containerSize, currentOffset, targetIndex);

        return this._offsetToSafeOffset(containerSize, offset);
    }

    getVisibleCellRange(containerSize, offset) {
        offset = this._safeOffsetToOffset(containerSize, offset);

        return this._cellSizeAndPositionManager.getVisibleCellRange(containerSize, offset);
    }

    resetCell(index) {
        this._cellSizeAndPositionManager.resetCell(index);
    }

    _getOffsetPercentage(containerSize, offset, totalSize) {
        return totalSize <= containerSize
            ? 0
            : offset / (totalSize - containerSize);
    }

    _offsetToSafeOffset(containerSize, offset) {
        const totalSize = this._cellSizeAndPositionManager.getTotalSize();
        const safeTotalSize = this.getTotalSize();

        if (totalSize === safeTotalSize) {
            return offset;
        }
        const offsetPercentage = this._getOffsetPercentage(containerSize, offset, totalSize);

        return Math.round(offsetPercentage * (safeTotalSize - containerSize));

    }

    _safeOffsetToOffset(containerSize, offset) {
        const totalSize = this._cellSizeAndPositionManager.getTotalSize();
        const safeTotalSize = this.getTotalSize();

        if (totalSize === safeTotalSize) {
            return offset;
        }
        const offsetPercentage = this._getOffsetPercentage(containerSize, offset, safeTotalSize);

        return Math.round(offsetPercentage * (totalSize - containerSize));

    }
}
