import { _global } from "../0.foundation";

// 线段树
const parent = function (node) {
    return Math.floor(node / 2);
};

const Int32Array = _global.Int32Array || function (size) {
    const xs = [];
    for (let i = size - 1; i >= 0; --i) {
        xs[i] = 0;
    }
    return xs;
};

const ceilLog2 = function (x) {
    let y = 1;
    while (y < x) {
        y *= 2;
    }
    return y;
};

export class PrefixIntervalTree {
    constructor(xs) {
        this._size = xs.length;
        this._half = ceilLog2(this._size);
        // _heap是一个_size两倍以上的堆
        this._heap = new Int32Array(2 * this._half);

        let i;
        // 初始化 >= _size 的堆空间, 即叶子节点
        for (i = 0; i < this._size; ++i) {
            this._heap[this._half + i] = xs[i];
        }
        // 初始化 < _size 的堆空间, 即非叶子节点，根节点包含整个区间
        for (i = this._half - 1; i > 0; --i) {
            this._heap[i] = this._heap[2 * i] + this._heap[2 * i + 1];
        }
    }

    static uniform(size, initialValue) {
        const xs = [];
        for (let i = size - 1; i >= 0; --i) {
            xs[i] = initialValue;
        }

        return new PrefixIntervalTree(xs);
    }

    static empty(size) {
        return PrefixIntervalTree.uniform(size, 0);
    }

    set(index, value) {
        let node = this._half + index;
        this._heap[node] = value;

        node = parent(node);
        for (; node !== 0; node = parent(node)) {
            this._heap[node] =
                this._heap[2 * node] + this._heap[2 * node + 1];
        }
    }

    get(index) {
        const node = this._half + index;
        return this._heap[node];
    }

    getSize() {
        return this._size;
    }

    /**
     * get(0) + get(1) + ... + get(end - 1).
     */
    sumUntil(end) {
        if (end === 0) {
            return 0;
        }

        let node = this._half + end - 1;
        let sum = this._heap[node];
        for (; node !== 1; node = parent(node)) {
            if (node % 2 === 1) {
                sum += this._heap[node - 1];
            }
        }

        return sum;
    }

    /**
     * get(0) + get(1) + ... + get(inclusiveEnd).
     */
    sumTo(inclusiveEnd) {
        return this.sumUntil(inclusiveEnd + 1);
    }

    /**
     * sum get(begin) + get(begin + 1) + ... + get(end - 1).
     */
    sum(begin, end) {
        return this.sumUntil(end) - this.sumUntil(begin);
    }

    /**
     * Returns the smallest i such that 0 <= i <= size and sumUntil(i) <= t, or
     * -1 if no such i exists.
     */
    greatestLowerBound(t) {
        if (t < 0) {
            return -1;
        }

        let node = 1;
        if (this._heap[node] <= t) {
            return this._size;
        }

        while (node < this._half) {
            const leftSum = this._heap[2 * node];
            if (t < leftSum) {
                node = 2 * node;
            } else {
                node = 2 * node + 1;
                t -= leftSum;
            }
        }

        return node - this._half;
    }

    /**
     * Returns the smallest i such that 0 <= i <= size and sumUntil(i) < t, or
     * -1 if no such i exists.
     */
    greatestStrictLowerBound(t) {
        if (t <= 0) {
            return -1;
        }

        let node = 1;
        if (this._heap[node] < t) {
            return this._size;
        }

        while (node < this._half) {
            const leftSum = this._heap[2 * node];
            if (t <= leftSum) {
                node = 2 * node;
            } else {
                node = 2 * node + 1;
                t -= leftSum;
            }
        }

        return node - this._half;
    }

    /**
     * Returns the smallest i such that 0 <= i <= size and t <= sumUntil(i), or
     * size + 1 if no such i exists.
     */
    leastUpperBound(t) {
        return this.greatestStrictLowerBound(t) + 1;
    }

    /**
     * Returns the smallest i such that 0 <= i <= size and t < sumUntil(i), or
     * size + 1 if no such i exists.
     */
    leastStrictUpperBound(t) {
        return this.greatestLowerBound(t) + 1;
    }


}
