import {
    isObject,
    UUID,
    extend,
    isEmpty,
    isArray,
    first,
    last,
    findIndex,
    isUndefined,
    isNull,
    each,
    deepClone,
    isEqual, some, every, clone
} from "../2.base";

export class Node {
    constructor(id) {
        if (isObject(id)) {
            extend(this, id);
        } else {
            this.id = id;
        }
        this.clear();
    }

    set(key, value) {
        if (isObject(key)) {
            extend(this, key);
            return;
        }
        this[key] = value;
    }

    get(key) {
        return this[key];
    }

    isLeaf() {
        return isEmpty(this.children);
    }

    getChildren() {
        return this.children;
    }

    getChildrenLength() {
        return this.children.length;
    }

    getFirstChild() {
        return first(this.children);
    }

    getLastChild() {
        return last(this.children);
    }

    setLeft(left) {
        this.left = left;
    }

    getLeft() {
        return this.left;
    }

    setRight(right) {
        this.right = right;
    }

    getRight() {
        return this.right;
    }

    setParent(parent) {
        this.parent = parent;
    }

    getParent() {
        return this.parent;
    }

    getChild(index) {
        return this.children[index];
    }

    getChildIndex(id) {
        return findIndex(this.children, function (i, ch) {
            return ch.get("id") === id;
        });
    }

    removeChild(id) {
        this.removeChildByIndex(this.getChildIndex(id));
    }

    removeChildByIndex(index) {
        const before = this.getChild(index - 1);
        const behind = this.getChild(index + 1);
        if (before != null) {
            before.setRight(behind || null);
        }
        if (behind != null) {
            behind.setLeft(before || null);
        }
        this.children.splice(index, 1);
    }

    removeAllChilds() {
        this.children = [];
    }

    addChild(child, index) {
        let cur = null;
        if (isUndefined(index)) {
            cur = this.children.length - 1;
        } else {
            cur = index - 1;
        }
        child.setParent(this);
        if (cur >= 0) {
            this.getChild(cur) && this.getChild(cur).setRight(child);
            child.setLeft(this.getChild(cur));
        }
        if (isUndefined(index)) {
            this.children.push(child);
        } else {
            this.children.splice(index, 0, child);
        }
    }

    equals(obj) {
        return this === obj || this.id === obj.id;
    }

    clear() {
        this.parent = null;
        this.left = null;
        this.right = null;
        this.children = [];
    }
}

export class Tree {
    constructor() {
        this.root = new Node(UUID());
    }

    addNode(node, newNode, index) {
        if (isNull(newNode)) {
            this.root.addChild(node, index);
        } else if (isNull(node)) {
            this.root.addChild(newNode, index);
        } else {
            node.addChild(newNode, index);
        }
    }

    isRoot(node) {
        return node === this.root;
    }

    getRoot() {
        return this.root;
    }

    clear() {
        this.root.clear();
    }

    initTree(nodes) {
        this.clear();
        const queue = new Set();
        each(nodes, (i, node) => {
            const n = new Node(node);
            n.set("data", node);
            this.addNode(n);
            queue.add(n);
        });
        queue.forEach(parent => {
            queue.delete(parent);
            const node = parent.get("data");
            each(node.children, (i, child) => {
                const n = new Node(child);
                n.set("data", child);
                queue.add(n);
                this.addNode(parent, n);
            });
        })
    }

    _toJSON(node) {
        const children = [];
        each(node.getChildren(), (i, child) => {
            children.push(this._toJSON(child));
        });
        return extend({
            id: node.id,
        }, deepClone(node.get("data")), (children.length > 0 ? {
            children: children,
        } : {}));
    }

    toJSON(node) {
        const result = [];
        each((node || this.root).getChildren(), (i, child) => {
            result.push(this._toJSON(child));
        });
        return result;
    }

    _toJSONWithNode(node) {
        const children = [];
        each(node.getChildren(), (i, child) => {
            children.push(this._toJSONWithNode(child));
        });
        return extend({
            id: node.id,
        }, deepClone(node.get("data")), {
            node: node,
        }, (children.length > 0 ? {
            children: children,
        } : {}));
    }

    toJSONWithNode(node) {
        const result = [];
        each((node || this.root).getChildren(), (i, child) => {
            result.push(this._toJSONWithNode(child));
        });
        return result;
    }

    search(root, target, param) {
        if (!(root instanceof Node)) {
            return this.search(this.root, root, target);
        }
        let next = null;

        if (isNull(target)) {
            return null;
        }
        if (isEqual(root[param || "id"], target)) {
            return root;
        }
        some(root.getChildren(), (i, child) => {
            next = this.search(child, target, param);
            if (null !== next) {
                return true;
            }
        });
        return next;
    }

    _traverse(node, callback) {
        let queue = [];
        queue.push(node);
        while (!isEmpty(queue)) {
            const temp = queue.shift();
            const b = callback && callback(temp);
            if (b === false) {
                break;
            }
            if (b === true) {
                continue;
            }
            if (temp != null) {
                queue.push(...temp.getChildren());
            }
        }
    }

    traverse(callback) {
        this._traverse(this.root, callback);
    }

    _recursion(node, route, callback) {
        return every(node.getChildren(), (i, child) => {
            const next = clone(route);
            next.push(child.id);
            const b = callback && callback(child, next);
            if (b === false) {
                return false;
            }
            if (b === true) {
                return true;
            }
            return this._recursion(child, next, callback);
        });
    }

    recursion(callback) {
        this._recursion(this.root, [], callback);
    }

    inOrderTraverse(callback) {
        this._inOrderTraverse(this.root, callback);
    }

    // 中序遍历(递归)
    _inOrderTraverse(node, callback) {
        if (node != null) {
            this._inOrderTraverse(node.getLeft());
            callback && callback(node);
            this._inOrderTraverse(node.getRight());
        }
    }

    // 中序遍历(非递归)
    nrInOrderTraverse(callback) {

        const stack = [];
        let node = this.root;
        while (node != null || !isEmpty(stack)) {
            while (node != null) {
                stack.push(node);
                node = node.getLeft();
            }
            node = stack.pop();
            callback && callback(node);
            node = node.getRight();
        }
    }

    preOrderTraverse(callback) {
        this._preOrderTraverse(this.root, callback);
    }

    // 先序遍历(递归)
    _preOrderTraverse(node, callback) {
        if (node != null) {
            callback && callback(node);
            this._preOrderTraverse(node.getLeft());
            this._preOrderTraverse(node.getRight());
        }
    }

    // 先序遍历（非递归）
    nrPreOrderTraverse(callback) {

        const stack = [];
        let node = this.root;

        while (node != null || !isEmpty(stack)) {

            while (node != null) {
                callback && callback(node);
                stack.push(node);
                node = node.getLeft();
            }
            node = stack.pop();
            node = node.getRight();
        }
    }

    postOrderTraverse(callback) {
        this._postOrderTraverse(this.root, callback);
    }

    // 后序遍历(递归)
    _postOrderTraverse(node, callback) {
        if (node != null) {
            this._postOrderTraverse(node.getLeft());
            this._postOrderTraverse(node.getRight());
            callback && callback(node);
        }
    }

    // 后续遍历(非递归)
    nrPostOrderTraverse(callback) {

        const stack = [];
        let node = this.root;
        let preNode = null;// 表示最近一次访问的节点

        while (node != null || !isEmpty(stack)) {

            while (node != null) {
                stack.push(node);
                node = node.getLeft();
            }

            node = last(stack);

            if (node.getRight() == null || node.getRight() === preNode) {
                callback && callback(node);
                node = stack.pop();
                preNode = node;
                node = null;
            } else {
                node = node.getRight();
            }
        }
    }

    static transformToArrayFormat(nodes, pId, childKey) {
        if (!nodes) return [];
        let r = [];
        childKey = childKey || "children";
        if (isArray(nodes)) {
            for (let i = 0, l = nodes.length; i < l; i++) {
                const node = clone(nodes[i]);
                node.pId = node.pId == null ? pId : node.pId;
                delete node.children;
                r.push(node);
                if (nodes[i][childKey]) {
                    r = r.concat(Tree.transformToArrayFormat(nodes[i][childKey], node.id));
                }
            }
        } else {
            const newNodes = clone(nodes);
            newNodes.pId = newNodes.pId == null ? pId : newNodes.pId;
            delete newNodes.children;
            r.push(newNodes);
            if (nodes[childKey]) {
                r = r.concat(Tree.transformToArrayFormat(nodes[childKey], newNodes.id));
            }
        }
        return r;
    }

    static arrayFormat(nodes, pId) {
        if (!nodes) {
            return [];
        }
        let r = [];
        if (isArray(nodes)) {
            for (let i = 0, l = nodes.length; i < l; i++) {
                const node = nodes[i];
                node.pId = node.pId == null ? pId : node.pId;
                r.push(node);
                if (nodes[i]["children"]) {
                    r = r.concat(Tree.arrayFormat(nodes[i]["children"], node.id));
                }
            }
        } else {
            const newNodes = nodes;
            newNodes.pId = newNodes.pId == null ? pId : newNodes.pId;
            r.push(newNodes);
            if (nodes["children"]) {
                r = r.concat(Tree.arrayFormat(nodes["children"], newNodes.id));
            }
        }
        return r;
    }

    static transformToTreeFormat(sNodes) {
        let i, l;
        if (!sNodes) {
            return [];
        }

        if (isArray(sNodes)) {
            const r = [];
            const tmpMap = {};
            for (i = 0, l = sNodes.length; i < l; i++) {
                if (isNull(sNodes[i].id)) {
                    return sNodes;
                }
                tmpMap[sNodes[i].id] = clone(sNodes[i]);
            }
            for (i = 0, l = sNodes.length; i < l; i++) {
                if (tmpMap[sNodes[i].pId] && sNodes[i].id !== sNodes[i].pId) {
                    if (!tmpMap[sNodes[i].pId].children) {
                        tmpMap[sNodes[i].pId].children = [];
                    }
                    tmpMap[sNodes[i].pId].children.push(tmpMap[sNodes[i].id]);
                } else {
                    r.push(tmpMap[sNodes[i].id]);
                }
                delete tmpMap[sNodes[i].id].pId;
            }
            return r;
        }
        return [sNodes];

    }

    static treeFormat(sNodes) {
        let i, l;
        if (!sNodes) {
            return [];
        }

        if (isArray(sNodes)) {
            const r = [];
            const tmpMap = {};
            for (i = 0, l = sNodes.length; i < l; i++) {
                if (isNull(sNodes[i].id)) {
                    return sNodes;
                }
                tmpMap[sNodes[i].id] = sNodes[i];
            }
            for (i = 0, l = sNodes.length; i < l; i++) {
                if (tmpMap[sNodes[i].pId] && sNodes[i].id !== sNodes[i].pId) {
                    if (!tmpMap[sNodes[i].pId].children) {
                        tmpMap[sNodes[i].pId].children = [];
                    }
                    tmpMap[sNodes[i].pId].children.push(tmpMap[sNodes[i].id]);
                } else {
                    r.push(tmpMap[sNodes[i].id]);
                }
            }
            return r;
        }
        return [sNodes];

    }

    static traversal(array, callback, pNode) {
        if (isNull(array)) {
            return;
        }
        some(array, (i, item) => {
            if (callback(i, item, pNode) === false) {
                return true;
            }
            this.traversal(item.children, callback, item);
        });
    }
}
