import { parseInt, parseFloat, isNull, isKey } from "../2.base";

export function isColor(color) {
    return color && (isRGBColor(color) || isHexColor(color));
}

export function isRGBColor(color) {
    if (!color) {
        return false;
    }

    return color.substr(0, 3) === "rgb";
}

export function isHexColor(color) {
    if (!color) {
        return false;
    }

    return color[0] === "#" && color.length === 7;
}

export function isDarkColor(hex) {
    if (!hex || !isHexColor(hex)) {
        return false;
    }
    const rgb = rgb2json(hex2rgb(hex));
    const grayLevel = Math.round(rgb.r * 0.299 + rgb.g * 0.587 + rgb.b * 0.114);
    if (grayLevel < 192/** 网上给的是140**/) {
        return true;
    }

    return false;
}

// 获取对比颜色
export function getContrastColor(color) {
    if (!color || !isColor(color)) {
        return "";
    }
    if (isDarkColor(color)) {
        return "#FFFFFF";
    }

    return "#091e40";
}

export function rgb2hex(rgbColour) {
    if (!rgbColour || rgbColour.substr(0, 3) !== "rgb") {
        return "";
    }
    const rgbValues = rgbColour.match(/\d+(\.\d+)?/g);
    const red = parseInt(rgbValues[0]);
    const green = parseInt(rgbValues[1]);
    const blue = parseInt(rgbValues[2]);

    const hexColour = `#${int2hex(red)}${int2hex(green)}${int2hex(blue)}`;

    return hexColour;
}

function _hue2rgb(m1, m2, h) {
    h = (h < 0) ? h + 1 : ((h > 1) ? h - 1 : h);
    if (h * 6 < 1) return m1 + (m2 - m1) * h * 6;
    if (h * 2 < 1) return m2;
    if (h * 3 < 2) return m1 + (m2 - m1) * (0.66666 - h) * 6;

    return m1;
}

export function hsl2rgb(hsl) {
    const h = hsl[0], s = hsl[1], l = hsl[2];
    const m2 = (l <= 0.5) ? l * (s + 1) : l + s - l * s;
    const m1 = l * 2 - m2;

    return [
        _hue2rgb(m1, m2, h + 0.33333),
        _hue2rgb(m1, m2, h),
        _hue2rgb(m1, m2, h - 0.33333)
    ];
}

export function rgb2hsl(rgb) {
    let h, s;
    const r = rgb[0], g = rgb[1], b = rgb[2];
    const min = Math.min(r, Math.min(g, b));
    const max = Math.max(r, Math.max(g, b));
    const delta = max - min;
    const l = (min + max) / 2;
    s = 0;
    if (l > 0 && l < 1) {
        s = delta / (l < 0.5 ? (2 * l) : (2 - 2 * l));
    }
    h = 0;
    if (delta > 0) {
        if (max === r && max !== g) h += (g - b) / delta;
        if (max === g && max !== b) h += (2 + (b - r) / delta);
        if (max === b && max !== r) h += (4 + (r - g) / delta);
        h /= 6;
    }

    return [h, s, l];
}

export function rgb2json(rgbColour) {
    if (!rgbColour) {
        return {};
    }
    if (!isRGBColor(rgbColour)) {
        return {};
    }
    const rgbValues = rgbColour.match(/\d+(\.\d+)?/g);

    return {
        r: parseInt(rgbValues[0]),
        g: parseInt(rgbValues[1]),
        b: parseInt(rgbValues[2]),
    };
}

export function rgba2json(rgbColour) {
    if (!rgbColour) {
        return {};
    }
    const rgbValues = rgbColour.match(/\d+(\.\d+)?/g);

    return {
        r: parseInt(rgbValues[0]),
        g: parseInt(rgbValues[1]),
        b: parseInt(rgbValues[2]),
        a: parseFloat(rgbValues[3]),
    };
}

export function json2rgb(rgb) {
    if (!isKey(rgb.r) || !isKey(rgb.g) || !isKey(rgb.b)) {
        return "";
    }

    return `rgb(${rgb.r},${rgb.g},${rgb.b})`;
}

export function json2rgba(rgba) {
    if (!isKey(rgba.r) || !isKey(rgba.g) || !isKey(rgba.b)) {
        return "";
    }

    return `rgba(${rgba.r},${rgba.g},${rgba.b},${rgba.a})`;
}

export function int2hex(strNum) {
    const hexdig = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f"];

    return `${hexdig[strNum >>> 4]}${hexdig[strNum & 15]}`;
}

export function hex2rgb(color) {
    if (!color) {
        return "";
    }
    if (!isHexColor(color)) {
        return color;
    }
    let tempValue = "rgb(", colorArray;

    if (color.length === 7) {
        colorArray = [
            parseInt(`0x${color.substring(1, 3)}`),
            parseInt(`0x${color.substring(3, 5)}`),
            parseInt(`0x${color.substring(5, 7)}`)
        ];
    } else if (color.length === 4) {
        colorArray = [
            parseInt(`0x${color.substring(1, 2)}`),
            parseInt(`0x${color.substring(2, 3)}`),
            parseInt(`0x${color.substring(3, 4)}`)
        ];
    }
    tempValue += `${colorArray[0]},`;
    tempValue += `${colorArray[1]},`;
    tempValue += `${colorArray[2]})`;

    return tempValue;
}

export function rgba2rgb(rgbColor, bgColor) {
    if (isNull(bgColor)) {
        bgColor = 1;
    }
    if (rgbColor.substr(0, 4) !== "rgba") {
        return "";
    }
    const rgbValues = rgbColor.match(/\d+(\.\d+)?/g);
    if (rgbValues.length < 4) {
        return "";
    }
    const R = parseFloat(rgbValues[0]);
    const G = parseFloat(rgbValues[1]);
    const B = parseFloat(rgbValues[2]);
    const A = parseFloat(rgbValues[3]);

    return `rgb(${Math.floor(255 * (bgColor * (1 - A)) + R * A)},${
        Math.floor(255 * (bgColor * (1 - A)) + G * A)},${
        Math.floor(255 * (bgColor * (1 - A)) + B * A)})`;
}

