import { EventListener } from "./eventlistener";
import { bind } from "../../2.base";
import { _global } from "../../0.foundation";

const cancelAnimationFrame =
    _global.cancelAnimationFrame ||
    _global.webkitCancelAnimationFrame ||
    _global.mozCancelAnimationFrame ||
    _global.oCancelAnimationFrame ||
    _global.msCancelAnimationFrame ||
    _global.clearTimeout;

const requestAnimationFrame = _global.requestAnimationFrame || _global.webkitRequestAnimationFrame || _global.mozRequestAnimationFrame || _global.oRequestAnimationFrame || _global.msRequestAnimationFrame || _global.setTimeout;

export class MouseMoveTracker {
    constructor(onMove, onMoveEnd, domNode) {
        this._isDragging = false;
        this._animationFrameID = null;
        this._domNode = domNode;
        this._onMove = onMove;
        this._onMoveEnd = onMoveEnd;
    
        this._onMouseMove = bind(this._onMouseMove, this);
        this._onMouseUp = bind(this._onMouseUp, this);
        this._didMouseMove = bind(this._didMouseMove, this);
    }

    captureMouseMoves(event) {
        if (!this._eventMoveToken && !this._eventUpToken) {
            this._eventMoveToken = EventListener.listen(
                this._domNode,
                "mousemove",
                this._onMouseMove
            );
            this._eventUpToken = EventListener.listen(
                this._domNode,
                "mouseup",
                this._onMouseUp
            );
        }

        if (!this._isDragging) {
            this._deltaX = 0;
            this._deltaY = 0;
            this._isDragging = true;
            this._x = event.clientX;
            this._y = event.clientY;
        }
        // event.preventDefault ? event.preventDefault() : (event.returnValue = false);
    }

    releaseMouseMoves() {
        if (this._eventMoveToken && this._eventUpToken) {
            this._eventMoveToken.remove();
            this._eventMoveToken = null;
            this._eventUpToken.remove();
            this._eventUpToken = null;
        }

        if (this._animationFrameID !== null) {
            cancelAnimationFrame(this._animationFrameID);
            this._animationFrameID = null;
        }

        if (this._isDragging) {
            this._isDragging = false;
            this._x = null;
            this._y = null;
        }
    }

    isDragging() /* boolean*/ {
        return this._isDragging;
    }

    _onMouseMove(/* object*/ event) {
        const x = event.clientX;
        const y = event.clientY;

        this._deltaX += (x - this._x);
        this._deltaY += (y - this._y);

        if (this._animationFrameID === null) {
            // The mouse may move faster then the animation frame does.
            // Use `requestAnimationFrame` to avoid over-updating.
            this._animationFrameID =
                requestAnimationFrame(() => this._didMouseMove(event));
        }

        this._x = x;
        this._y = y;
        event.preventDefault ? event.preventDefault() : (event.returnValue = false);
    }

    _didMouseMove(event) {
        this._animationFrameID = null;
        this._onMove(this._deltaX, this._deltaY, event);
        this._deltaX = 0;
        this._deltaY = 0;
    }

    _onMouseUp(event) {
        if (this._animationFrameID) {
            this._didMouseMove(event);
        }
        this._onMoveEnd(event);
    }
}
