import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, makeArray, each, isArray } from "../../2.base";
import { Widget } from "../../4.widget";
import { _lazyCreateWidget } from "../../5.inject";

/**
 * 分隔容器的控件，按照宽度和高度所占比平分整个容器
 *
 * @class DivisionLayout
 * @extends Layout
 */
@shortcut()
export class DivisionLayout extends Layout {
    static xtype = "bi.division";
    
    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-division",
            columns: null,
            rows: null,
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(items) {
        const o = this.options;
        const rows = o.rows || o.items.length, columns = o.columns || ((o.items[0] && o.items[0].length) | 0);
        const map = makeArray(rows), widths = {}, heights = {};

        function firstElement (item, cls) {
            item.addClass(cls);

            return item;
        }

        function firstObject (item, cls) {
            item.cls = (item.cls || "") + cls;

            return item;
        }

        function first (item, cls) {
            if (item instanceof Widget) {
                firstElement(item.element, cls);
            } else if (item.el instanceof Widget) {
                firstElement(item.el.element, cls);
            } else if (item.el) {
                firstObject(item.el, cls);
            } else {
                firstObject(item, cls);
            }
        }

        each(map, i => {
            map[i] = makeArray(columns);
        });
        each(items, (i, item) => {
            if (isArray(item)) {
                each(item, (j, el) => {
                    widths[i] = (widths[i] || 0) + item.width;
                    heights[j] = (heights[j] || 0) + item.height;
                    map[i][j] = el;
                });

                return;
            }
            widths[item.row] = (widths[item.row] || 0) + item.width;
            heights[item.column] = (heights[item.column] || 0) + item.height;
            map[item.row][item.column] = item;
        });
        for (let i = 0; i < rows; i++) {
            let totalW = 0;
            for (let j = 0; j < columns; j++) {
                let w;
                if (!map[i][j]) {
                    throw new Error(`item(${i}${j}) must exist`, map);
                }
                if (!this.hasWidget(this._getChildName(`${i}_${j}`))) {
                    w = _lazyCreateWidget(map[i][j]);
                    this.addWidget(this._getChildName(`${i}_${j}`), w);
                } else {
                    w = this.getWidgetByName(this._getChildName(`${i}_${j}`));
                }
                const left = totalW * 100 / widths[i];
                w.element.css({ position: "absolute", left: `${left}%` });
                if (j > 0) {
                    const lastW = this.getWidgetByName(this._getChildName(`${i}_${j - 1}`));
                    lastW.element.css({ right: `${100 - left}%` });
                }
                if (j === o.columns - 1) {
                    w.element.css({ right: "0%" });
                }
                first(w, this.getRowColumnCls(i, j, rows - 1, columns - 1));
                totalW += map[i][j].width;
            }
        }
        for (let j = 0; j < o.columns; j++) {
            let totalH = 0;
            for (let i = 0; i < o.rows; i++) {
                const w = this.getWidgetByName(this._getChildName(`${i}_${j}`));
                const top = totalH * 100 / heights[j];
                w.element.css({ top: `${top}%` });
                if (i > 0) {
                    const lastW = this.getWidgetByName(this._getChildName(`${i - 1}_${j}`));
                    lastW.element.css({ bottom: `${100 - top}%` });
                }
                if (i === o.rows - 1) {
                    w.element.css({ bottom: "0%" });
                }
                totalH += map[i][j].height;
            }
        }
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
