import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, isNotNull, isNumber } from "../../2.base";
import { pixFormat } from "../../constant";

/**
 * 靠左对齐的自由浮动布局
 * @class FloatLeftLayout
 * @extends Layout
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Number} [hgap=0] 水平间隙
 * @cfg {Number} [vgap=0] 垂直间隙
 */
@shortcut()
export class FloatLeftLayout extends Layout {
    static xtype = "bi.left";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-left clearfix",
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        if (o.innerHgap !== 0) {
            this.element.css({
                paddingLeft: this._optimiseGap(o.innerHgap),
                paddingRight: this._optimiseGap(o.innerHgap),
            });
        }
        if (o.innerVgap !== 0) {
            this.element.css({
                paddingTop: this._optimiseGap(o.innerVgap),
                paddingBottom: this._optimiseGap(o.innerVgap),
            });
        }
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        w.element.css({ position: "relative", "float": "left" });
        if (isNotNull(item.left)) {
            w.element.css({ left: isNumber(item.left) ? this._optimiseGap(item.left) : item.left });
        }
        if (isNotNull(item.right)) {
            w.element.css({ right: isNumber(item.right) ? this._optimiseGap(item.right) : item.right });
        }
        if (isNotNull(item.top)) {
            w.element.css({ top: isNumber(item.top) ? this._optimiseGap(item.top) : item.top });
        }
        if (isNotNull(item.bottom)) {
            w.element.css({ bottom: isNumber(item.bottom) ? this._optimiseGap(item.bottom) : item.bottom });
        }
        if (o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) !== 0) {
            const top = o.vgap / 2 + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item);
            w.element.css({
                "margin-top": this._optimiseGap(top),
            });
        }
        if (o.hgap + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
            const left = o.hgap / 2 + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item);
            w.element.css({
                "margin-left": this._optimiseGap(left),
            });
        }
        if (o.hgap + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
            const right = o.hgap / 2 + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item);
            w.element.css({
                "margin-right": this._optimiseGap(right),
            });
        }
        if (o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) !== 0) {
            const bottom = o.vgap / 2 + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item);
            w.element.css({
                "margin-bottom": this._optimiseGap(bottom),
            });
        }
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}

/**
 * 靠右对齐的自由浮动布局
 * @class FloatRightLayout
 * @extends Layout
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Number} [hgap=0] 水平间隙
 * @cfg {Number} [vgap=0] 垂直间隙
 */
@shortcut()
export class FloatRightLayout extends Layout {
    static xtype = "bi.right";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-right clearfix",
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        if (o.innerHgap !== 0) {
            this.element.css({
                paddingLeft: this._optimiseGap(o.innerHgap),
                paddingRight: this._optimiseGap(o.innerHgap),
            });
        }
        if (o.innerVgap !== 0) {
            this.element.css({
                paddingTop: this._optimiseGap(o.innerVgap),
                paddingBottom: this._optimiseGap(o.innerVgap),
            });
        }
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        w.element.css({ position: "relative", "float": "right" });
        if (isNotNull(item.left)) {
            w.element.css({ left: pixFormat(item.left) });
        }
        if (isNotNull(item.right)) {
            w.element.css({ right: pixFormat(item.right) });
        }
        if (isNotNull(item.top)) {
            w.element.css({ top: pixFormat(item.top) });
        }
        if (isNotNull(item.bottom)) {
            w.element.css({ bottom: pixFormat(item.bottom) });
        }
        if (o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) !== 0) {
            const top = o.vgap / 2 + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item);
            w.element.css({
                "margin-top": this._optimiseGap(top),
            });
        }
        if (o.hgap + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
            const left = o.hgap / 2 + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item);
            w.element.css({
                "margin-left": this._optimiseGap(left),
            });
        }
        if (o.hgap + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
            const right = o.hgap / 2 + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item);
            w.element.css({
                "margin-right": this._optimiseGap(right),
            });
        }
        if (o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) !== 0) {
            const bottom = o.vgap / 2 + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item);
            w.element.css({
                "margin-bottom": this._optimiseGap(bottom),
            });
        }
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
