import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, isNull } from "../../2.base";
import { pixFormat, HorizontalAlign, VerticalAlign } from "../../constant";

/**
 * 内联布局
 * @class InlineLayout
 * @extends Layout
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Number} [hgap=0] 水平间隙
 * @cfg {Number} [vgap=0] 垂直间隙
 */
@shortcut()
export class InlineLayout extends Layout {
    static xtype = "bi.inline";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-i",
            horizontalAlign: HorizontalAlign.Left,
            verticalAlign: VerticalAlign.Top,
            columnSize: [],
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        this.element.css({
            textAlign: o.horizontalAlign,
        });
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        let columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
        if (o.columnSize.length > 0) {
            if (item.width >= 1 && o.columnSize[i] >= 1 && o.columnSize[i] !== item.width) {
                columnSize = null;
            }
        }
        if (columnSize > 0) {
            w.element.width(columnSize < 1 ? (`${(columnSize * 100).toFixed(1)}%`) : pixFormat(columnSize));
        }
        w.element.css({
            position: "relative",
            "vertical-align": o.verticalAlign,
        });
        w.element.addClass("i-item");
        if (columnSize === "fill" || columnSize === "") {
            let length = 0, gap = o.hgap + o.innerHgap;
            let fillCount = 0, autoCount = 0;
            for (let k = 0, len = o.columnSize.length || o.items.length; k < len; k++) {
                let cz = o.columnSize.length > 0 ? o.columnSize[k] : o.items[k].width;
                if (cz === "fill") {
                    fillCount++;
                    cz = 0;
                } else if (cz === "" || isNull(cz)) {
                    autoCount++;
                    cz = 0;
                }
                gap += o.hgap + o.lgap + o.rgap + this._optimiseItemLgap(o.items[k]) + this._optimiseItemRgap(o.items[k]) + this._optimiseItemHgap(o.items[k]);
                length += cz;
            }
            length = length > 0 && length < 1 ? `${(length * 100).toFixed(1)}%` : pixFormat(length);
            gap = gap > 0 && gap < 1 ? `${(gap * 100).toFixed(1)}%` : pixFormat(gap);
            if (columnSize === "fill") {
                w.element.css("min-width", `calc((100% - ${length} - ${gap})${fillCount > 1 ? `/${fillCount}` : ""})`);
            }
            if (o.horizontalAlign === HorizontalAlign.Stretch || !(o.scrollable === true || o.scrollx === true)) {
                if (columnSize === "fill") {
                    w.element.css("max-width", `calc((100% - ${length} - ${gap})${fillCount > 1 ? `/${fillCount}` : ""})`);
                } else {
                    w.element.css("max-width", `calc((100% - ${length} - ${gap})${autoCount > 1 ? `/${autoCount}` : ""})`);
                }
            }
        }
        this._handleGap(w, item, i);
        if (o.verticalAlign === VerticalAlign.Stretch && isNull(item.height)) {
            const top = o.innerVgap + o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item),
                bottom = o.innerVgap + o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item);
            const gap = (top + bottom) > 0 && (top + bottom) < 1 ? `${((top + bottom) * 100).toFixed(1)}%` : pixFormat(top + bottom);
            w.element.css("height", `calc(100% - ${gap})`);
        }
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
