import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, compact, each, isEmptyObject, isNumber, isNull, any, backAny } from "../../2.base";
import { _lazyCreateWidget } from "../../5.inject";
import { HorizontalAlign, VerticalAlign } from "../../constant";

/**
 * 水平tape布局
 * @class HTapeLayout
 * @extends Layout
 */
@shortcut()
export class HTapeLayout extends Layout {
    static xtype = "bi.htape";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-h-tape",
            verticalAlign: VerticalAlign.Top,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            columnSize: [],
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(items) {
        const o = this.options;
        items = compact(items);
        each(items, (i, item) => {
            let w;
            if (isEmptyObject(item)) {
                return;
            }
            if (!this.hasWidget(this._getChildName(i))) {
                w = _lazyCreateWidget(item);
                this.addWidget(this._getChildName(i), w);
            } else {
                w = this.getWidgetByName(this._getChildName(i));
            }
            let columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (o.columnSize.length > 0) {
                if (item.width >= 1 && o.columnSize[i] >= 1 && o.columnSize[i] !== item.width) {
                    columnSize = item.width;
                }
            }
            w.element.css({
                position: "absolute",
                top: this._optimiseGap(this._optimiseItemTgap(item) + this._optimiseItemVgap(item) + o.innerVgap + o.vgap + o.tgap),
                bottom: this._optimiseGap(this._optimiseItemBgap(item) + this._optimiseItemVgap(item) + o.innerVgap + o.vgap + o.bgap),
                width: isNumber(columnSize) ? this._optimiseGap(columnSize) : "",
            });
            if (o.verticalAlign === VerticalAlign.Middle) {
                w.element.css({
                    marginTop: "auto",
                    marginBottom: "auto",
                });
            } else if (o.verticalAlign === VerticalAlign.Bottom) {
                w.element.css({
                    marginTop: "auto",
                });
            }
        });

        const left = {}, right = {};
        left[0] = o.innerHgap;
        right[items.length - 1] = o.innerHgap;

        any(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const w = this.getWidgetByName(this._getChildName(i));
            let columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (o.columnSize.length > 0) {
                if (item.width >= 1 && o.columnSize[i] >= 1 && o.columnSize[i] !== item.width) {
                    columnSize = item.width;
                }
            }
            if (isNull(left[i])) {
                const preColumnSize = o.columnSize.length > 0 ? o.columnSize[i - 1] : items[i - 1].width;
                left[i] = left[i - 1] + preColumnSize + this._optimiseItemLgap(items[i - 1]) + this._optimiseItemRgap(items[i - 1]) + 2 * this._optimiseItemHgap(items[i - 1]) + o.hgap + o.lgap + o.rgap;
            }
            w.element.css({
                left: this._optimiseGap(left[i] + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) + o.hgap + o.lgap),
            });

            if (isNull(columnSize) || columnSize === "" || columnSize === "fill") {
                return true;
            }
        });
        backAny(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const w = this.getWidgetByName(this._getChildName(i));
            const columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (isNull(right[i])) {
                const nextColumnSize = o.columnSize.length > 0 ? o.columnSize[i + 1] : items[i + 1].width;
                right[i] = right[i + 1] + nextColumnSize + this._optimiseItemLgap(items[i + 1]) + this._optimiseItemRgap(items[i + 1]) + 2 * this._optimiseItemHgap(items[i + 1]) + o.hgap + o.lgap + o.rgap;
            }
            w.element.css({
                right: this._optimiseGap(right[i] + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) + o.hgap + o.rgap),
            });

            if (isNull(columnSize) || columnSize === "" || columnSize === "fill") {
                return true;
            }
        });
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(items) {
        super.populate(...arguments);
        this._mount();
    }
}

/**
 * 垂直tape布局
 * @class VTapeLayout
 * @extends Layout
 */
@shortcut()
export class VTapeLayout extends Layout {
    static xtype = "bi.vtape";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-v-tape",
            horizontalAlign: HorizontalAlign.Left,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            rowSize: [],
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        this.populate(this.options.items);
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(items) {
        const o = this.options;
        items = compact(items);
        each(items, (i, item) => {
            let w;
            if (isEmptyObject(item)) {
                return;
            }
            if (!this.hasWidget(this._getChildName(i))) {
                w = _lazyCreateWidget(item);
                this.addWidget(this._getChildName(i), w);
            } else {
                w = this.getWidgetByName(this._getChildName(i));
            }
            let rowSize = o.rowSize.length > 0 ? o.rowSize[i] : item.height;
            if (o.rowSize.length > 0) {
                if (item.height >= 1 && o.rowSize[i] >= 1 && o.rowSize[i] !== item.height) {
                    rowSize = item.height;
                }
            }
            w.element.css({
                position: "absolute",
                left: this._optimiseGap(this._optimiseItemLgap(item) + this._optimiseItemHgap(item) + o.innerHgap + o.hgap + o.lgap),
                right: this._optimiseGap(this._optimiseItemRgap(item) + this._optimiseItemHgap(item) + o.innerHgap + o.hgap + o.rgap),
                height: isNumber(rowSize) ? this._optimiseGap(rowSize) : "",
            });
            if (o.horizontalAlign === HorizontalAlign.Center) {
                w.element.css({
                    marginLeft: "auto",
                    marginRight: "auto",
                });
            } else if (o.horizontalAlign === HorizontalAlign.Right) {
                w.element.css({
                    marginLeft: "auto",
                });
            }
        });

        const top = {}, bottom = {};
        top[0] = o.innerVgap;
        bottom[items.length - 1] = o.innerVgap;

        any(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const w = this.getWidgetByName(this._getChildName(i));
            let rowSize = o.rowSize.length > 0 ? o.rowSize[i] : item.height;
            if (o.rowSize.length > 0) {
                if (item.height >= 1 && o.rowSize[i] >= 1 && o.rowSize[i] !== item.height) {
                    rowSize = item.height;
                }
            }
            if (isNull(top[i])) {
                const preRowSize = o.rowSize.length > 0 ? o.rowSize[i - 1] : items[i - 1].height;
                top[i] = top[i - 1] + preRowSize + this._optimiseItemTgap(items[i - 1]) + this._optimiseItemBgap(items[i - 1]) + 2 * this._optimiseItemVgap(items[i - 1]) + o.vgap + o.tgap + o.bgap;
            }
            w.element.css({
                top: this._optimiseGap(top[i] + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) + o.vgap + o.tgap),
            });

            if (isNull(rowSize) || rowSize === "" || rowSize === "fill") {
                return true;
            }
        });
        backAny(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const w = this.getWidgetByName(this._getChildName(i));
            const rowSize = o.rowSize.length > 0 ? o.rowSize[i] : item.height;
            if (isNull(bottom[i])) {
                const nextRowSize = o.rowSize.length > 0 ? o.rowSize[i + 1] : items[i + 1].height;
                bottom[i] = bottom[i + 1] + nextRowSize + this._optimiseItemTgap(items[i + 1]) + this._optimiseItemBgap(items[i + 1]) + 2 * this._optimiseItemVgap(items[i + 1]) + o.vgap + o.tgap + o.bgap;
            }
            w.element.css({
                bottom: this._optimiseGap(bottom[i] + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) + o.vgap + o.bgap),
            });

            if (isNull(rowSize) || rowSize === "" || rowSize === "fill") {
                return true;
            }
        });
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
