import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, isOdd, some, isNumber, isNull } from "../../2.base";
import { Widget } from "../../4.widget";
import { _lazyCreateWidget } from "../../5.inject";
import { HorizontalAlign, VerticalAlign } from "../../constant";

/**
 * td布局
 * @class TdLayout
 * @extends Layout
 */
@shortcut()
export class TdLayout extends Layout {
    static xtype = "bi.td";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-td",
            columnSize: [],
            rowSize: [],
            verticalAlign: VerticalAlign.Middle,
            horizontalAlign: HorizontalAlign.Stretch,
            hgap: 0,
            vgap: 0,
            tgap: 0,
            bgap: 0,
            lgap: 0,
            rgap: 0,
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        this.$table = Widget._renderEngine.createElement("<table>").attr({ cellspacing: 0, cellpadding: 0 }).css({
            position: "relative",
            width: (o.horizontalAlign === HorizontalAlign.Center || o.horizontalAlign === HorizontalAlign.Stretch) ? "100%" : "auto",
            height: (o.verticalAlign !== VerticalAlign.Top) ? "100%" : "auto",
            "border-spacing": "0px",
            border: "none",
            "border-collapse": "separate",
        });
        this.rows = 0;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(idx, arr) {
        const o = this.options;

        function firstElement (item, row, col) {
            if (row === 0) {
                item.addClass("first-row");
            }
            if (col === 0) {
                item.addClass("first-col");
            }
            item.addClass(isOdd(row + 1) ? "odd-row" : "even-row");
            item.addClass(isOdd(col + 1) ? "odd-col" : "even-col");
            item.addClass("center-element");
        }

        function firstObject (item, row, col) {
            let cls = "";
            if (row === 0) {
                cls += " first-row";
            }
            if (col === 0) {
                cls += " first-col";
            }
            isOdd(row + 1) ? (cls += " odd-row") : (cls += " even-row");
            isOdd(col + 1) ? (cls += " odd-col") : (cls += " even-col");
            item.cls = `${(item.cls || "") + cls} center-element`;
        }

        function first (item, row, col) {
            if (item instanceof Widget) {
                firstElement(item.element, row, col);
            } else if (item.el instanceof Widget) {
                firstElement(item.el.element, row, col);
            } else if (item.el) {
                firstObject(item.el, row, col);
            } else {
                firstObject(item, row, col);
            }
        }

        const height = isNumber(o.rowSize) ? this._optimiseGap(o.rowSize) : (o.rowSize[idx] === "" ? this._optimiseGap(1) : this._optimiseGap(o.rowSize[idx]));
        const tr = _lazyCreateWidget({
            type: "bi.default",
            tagName: "tr",
            height,
            css: {
                "max-height": height,
                "min-height": height,
            },
        });

        for (let i = 0; i < arr.length; i++) {
            const w = _lazyCreateWidget(arr[i]);
            if (o.verticalAlign === VerticalAlign.Stretch) {
                const top = o.vgap + o.tgap + this._optimiseItemTgap(arr[i]) + this._optimiseItemVgap(arr[i]),
                    bottom = o.vgap + o.bgap + this._optimiseItemBgap(arr[i]) + this._optimiseItemVgap(arr[i]);
                w.element.css("height", `calc(100% - ${this._optimiseGap(top + bottom)})`);
            }
            w.element.css({ position: "relative", top: "0", left: "0", margin: "0px auto" });
            const item = arr[i];
            this._handleGap(w, item, i);
            first(w, this.rows++, i);
            let width = "";
            const columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (columnSize > 0) {
                width = this._optimiseGap(columnSize + (i === 0 ? o.hgap : 0) + o.hgap + o.lgap + o.rgap);
            }
            function hasFill() {
                if (o.columnSize.length > 0) {
                    return o.columnSize.indexOf("fill") >= 0;
                }
                
                return some(arr, (i, item) => {
                    if (item.width === "fill") {
                        return true;
                    }
                });
            }
            if ((isNull(columnSize) || columnSize === "") && hasFill()) {
                width = 2;
            }
            const td = _lazyCreateWidget({
                type: "bi.default",
                width,
                tagName: "td",
                items: [w],
            });
            // 对于表现为td的元素设置最大宽度，有几点需要注意
            // 1、由于直接对td设置最大宽度是在规范中未定义的, 所以要使用类似td:firstChild来迂回实现
            // 2、不能给多个td设置最大宽度，这样只会平分宽度
            // 3、多百分比宽度就算了
            if (columnSize > 0) {
                td.element.css({
                    "max-width": width,
                    "min-width": width,
                });
            }
            td.element.css({
                position: "relative",
                "vertical-align": o.verticalAlign,
                margin: "0",
                padding: "0",
                border: "none",
            });
            tr.addItem(td);
        }
        this.addWidget(this._getChildName(idx), tr);
        
        return tr;
    }

    appendFragment(frag) {
        this.$table.append(frag);
        this.element.append(this.$table);
    }

    resize() {
        // console.log("td布局不需要resize");
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
