import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, makeArray, isNumber, isNull } from "../../2.base";
import { Widget } from "../../4.widget";
import { _lazyCreateWidget } from "../../5.inject";

/**
 *
 * @class WindowLayout
 * @extends Layout
 */
@shortcut()
export class WindowLayout extends Layout {
    static xtype = "bi.window";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-window",
            columns: 3,
            rows: 2,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            columnSize: [],
            rowSize: [],
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(items) {
        const o = this.options;
        if (isNumber(o.rowSize)) {
            o.rowSize = makeArray(o.items.length, 1 / o.items.length);
        }
        if (isNumber(o.columnSize)) {
            o.columnSize = makeArray(o.items[0].length, 1 / o.items[0].length);
        }

        function firstElement (item, cls) {
            item.addClass(cls);

            return item;
        }

        function firstObject (item, cls) {
            item.cls = (item.cls || "") + cls;

            return item;
        }

        function first (item, row, col) {
            if (item instanceof Widget) {
                firstElement(item.element, row, col);
            } else if (item.el instanceof Widget) {
                firstElement(item.el.element, row, col);
            } else if (item.el) {
                firstObject(item.el, row, col);
            } else {
                firstObject(item, row, col);
            }
        }

        for (let i = 0; i < o.rows; i++) {
            for (let j = 0; j < o.columns; j++) {
                if (!o.items[i][j]) {
                    throw new Error("create items error", o.items);
                }
                if (!this.hasWidget(this._getChildName(`${i}_${j}`))) {
                    const w = _lazyCreateWidget(o.items[i][j]);
                    w.element.css({ position: "absolute" });
                    this.addWidget(this._getChildName(`${i}_${j}`), w);
                }
            }
        }
        const left = {}, right = {}, top = {}, bottom = {};
        left[0] = 0;
        top[0] = 0;
        right[o.columns - 1] = 0;
        bottom[o.rows - 1] = 0;
        // 从上到下
        for (let i = 0; i < o.rows; i++) {
            for (let j = 0; j < o.columns; j++) {
                const wi = this.getWidgetByName(this._getChildName(`${i}_${j}`));
                if (isNull(top[i])) {
                    top[i] = top[i - 1] + (o.rowSize[i - 1] < 1 ? o.rowSize[i - 1] : o.rowSize[i - 1] + o.vgap + o.bgap);
                }
                const t = this._optimiseGap(top[i] + o.vgap + o.tgap);
                let h = "";
                if (isNumber(o.rowSize[i])) {
                    h = this._optimiseGap(o.rowSize[i]);
                }
                wi.element.css({ top: t, height: h });
                first(wi, this.getRowColumnCls(i, j, o.rows - 1, o.columns - 1));
            }
            if (!isNumber(o.rowSize[i])) {
                break;
            }
        }
        // 从下到上
        for (let i = o.rows - 1; i >= 0; i--) {
            for (let j = 0; j < o.columns; j++) {
                const wi = this.getWidgetByName(this._getChildName(`${i}_${j}`));
                if (isNull(bottom[i])) {
                    bottom[i] = bottom[i + 1] + (o.rowSize[i + 1] < 1 ? o.rowSize[i + 1] : o.rowSize[i + 1] + o.vgap + o.tgap);
                }
                const b = this._optimiseGap(bottom[i] + o.vgap + o.bgap);
                let h = "";
                if (isNumber(o.rowSize[i])) {
                    h = this._optimiseGap(o.rowSize[i]);
                }
                wi.element.css({ bottom: b, height: h });
                first(wi, this.getRowColumnCls(i, j, o.rows - 1, o.columns - 1));
            }
            if (!isNumber(o.rowSize[i])) {
                break;
            }
        }
        // 从左到右
        for (let j = 0; j < o.columns; j++) {
            for (let i = 0; i < o.rows; i++) {
                const wi = this.getWidgetByName(this._getChildName(`${i}_${j}`));
                if (isNull(left[j])) {
                    left[j] = left[j - 1] + (o.columnSize[j - 1] < 1 ? o.columnSize[j - 1] : o.columnSize[j - 1] + o.hgap + o.rgap);
                }
                const l = this._optimiseGap(left[j] + o.hgap + o.lgap);
                let w = "";
                if (isNumber(o.columnSize[j])) {
                    w = this._optimiseGap(o.columnSize[j]);
                }
                wi.element.css({ left: l, width: w });
                first(wi, this.getRowColumnCls(i, j, o.rows - 1, o.columns - 1));
            }
            if (!isNumber(o.columnSize[j])) {
                break;
            }
        }
        // 从右到左
        for (let j = o.columns - 1; j >= 0; j--) {
            for (let i = 0; i < o.rows; i++) {
                const wi = this.getWidgetByName(this._getChildName(`${i}_${j}`));
                if (isNull(right[j])) {
                    right[j] = right[j + 1] + (o.columnSize[j + 1] < 1 ? o.columnSize[j + 1] : o.columnSize[j + 1] + o.hgap + o.lgap);
                }
                const r = this._optimiseGap(right[j] + o.hgap + o.rgap);
                let w = "";
                if (isNumber(o.columnSize[j])) {
                    w = this._optimiseGap(o.columnSize[j]);
                }
                wi.element.css({ right: r, width: w });
                first(wi, this.getRowColumnCls(i, j, o.rows - 1, o.columns - 1));
            }
            if (!isNumber(o.columnSize[j])) {
                break;
            }
        }
    }

    resize() {
        // console.log("window布局不需要resize");
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
