import {
    shortcut,
    Widget,
    debounce,
    bind,
    extend,
    Controller,
    contains,
    isKey,
    Events,
    trim,
    replaceAll,
    KeyCode,
    endWith,
    BlankSplitChar
} from "@/core";
import { Editor, TextAreaEditor } from "@/base";
import { StateEditor } from "@/case";

@shortcut()
export class SelectPatchEditor extends Widget {
    static xtype = "bi.select_patch_editor";

    props = { baseCls: "bi-patch-select-editor", height: 24 };

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";

    render() {
        const self = this,
            o = this.options;

        const debounceInputChange = debounce(bind(this._dealChange, this), 300);

        return extend(
            {
                type: StateEditor.xtype,
                ref(_ref) {
                    self.editor = _ref;
                },
                hgap: o.hgap,
                vgap: o.vgap,
                lgap: o.lgap,
                rgap: o.rgap,
                tgap: o.tgap,
                bgap: o.bgap,
                height: o.height,
                watermark: o.watermark,
                allowBlank: true,
                value: o.value,
                defaultText: o.defaultText,
                text: o.text,
                tipType: o.tipType,
                warningTitle: o.warningTitle,
                el: {
                    type: TextAreaEditor.xtype,
                    scrolly: false,
                    validationChecker() {
                        return true;
                    },
                    throttle: true,
                },
                listeners: [
                    {
                        eventName: Controller.EVENT_CHANGE,
                        action(type, v) {
                            if (contains(v, "\n")) {
                                self._dealChange(type, v);
                            } else {
                                debounceInputChange(type, v);
                            }
                        },
                    },
                    {
                        eventName: Editor.EVENT_KEY_DOWN,
                        action(keyCode) {
                            if (keyCode === KeyCode.ENTER) {
                                self._clearSplitValue();
                            }
                        },
                    },
                    {
                        eventName: Editor.EVENT_FOCUS,
                        action() {
                            self.fireEvent(SelectPatchEditor.EVENT_FOCUS, arguments);
                        },
                    },
                    {
                        eventName: Editor.EVENT_BLUR,
                        action() {
                            self._start = false;
                            self.fireEvent(SelectPatchEditor.EVENT_BLUR, arguments);
                        },
                    }
                ],
            },
            o.el
        );
    }

    _clearSplitValue() {
        this.editor.setValue("");
    }

    _dealChange(type, v) {
        let value = "";
        if (v !== this.editor.getValue()) {
            return;
        }
        if (isKey(v)) {
            value = this._formatText(v);
        }
        if (type === Events.CHANGE) {
            this._setValue(value);
            if (this._trimValue(value) !== "") {
                if (
                    !this._start ||
                    !isKey(this._lastValue) ||
                    (this._pause === true && this._trimValue(this._lastValue) !== this._trimValue(value))
                ) {
                    this._start = true;
                    this._pause = false;
                    this.fireEvent(Controller.EVENT_CHANGE, Events.STARTEDIT, this.getValue(), this);
                }
            }
            if (this._trimValue(this._lastValue) !== this._trimValue(value)) {
                this.fireEvent(Controller.EVENT_CHANGE, arguments);
            }
            if (endWith(value, BlankSplitChar)) {
                this._pause = true;
                this.fireEvent(Controller.EVENT_CHANGE, Events.PAUSE, "", this);
            }
        }
        if (type === Events.EMPTY || type === Events.STOPEDIT) {
            this.fireEvent(Controller.EVENT_CHANGE, Events.EMPTY);
        }
        this._lastValue = value;
    }

    _trimValue(v) {
        return trim(replaceAll(v || "", BlankSplitChar, ""));
    }

    _formatText(v) {
        return replaceAll(v || "", "\n", BlankSplitChar);
    }

    setWaterMark(v) {
        this.editor.setWaterMark(v);
    }

    doRedMark() {
        this.editor.doRedMark(...arguments);
    }

    unRedMark() {
        this.editor.unRedMark(...arguments);
    }

    doHighLight() {
        this.editor.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }

    focus() {
        this.editor.focus();
    }

    blur() {
        this.editor.blur();
    }

    _setValue(v) {
        this.editor.setValue(this._formatText(v));
    }

    _showInput() {
        this.editor.visible();
        this.text.invisible();
    }

    _showHint() {
        this.editor.invisible();
        this.text.visible();
    }

    isValid() {
        return this.editor.isValid();
    }

    setErrorText(text) {
        this.editor.setErrorText(text);
    }

    getErrorText() {
        return this.editor.getErrorText();
    }

    isEditing() {
        return this.editor.isEditing();
    }

    getLastValidValue() {
        return this.editor.getLastValidValue();
    }

    getLastChangedValue() {
        return this.editor.getLastChangedValue();
    }

    setValue(v) {
        this._setValue(v);
        this._lastValue = this._trimValue(v);
    }

    getValue() {
        return trim(this.editor.getValue());
    }

    getState() {
        return this.editor.getState();
    }

    setState(v) {
        this.editor.setState(v);
    }

    setTipType(v) {
        this.editor.setTipType(v);
    }

    getText() {
        return this.editor.getText();
    }
}
