import {
    shortcut,
    Widget,
    extend,
    emptyFn,
    i18nText,
    createWidget,
    isNotNull,
    isEmptyArray,
    size,
    each,
    BlankSplitChar,
    Selection,
    PasteLimit,
} from "@/core";
import { MultiSelectEditor } from "./editor.multiselect";
import { Searcher } from "@/base";
import { MultiSelectSearchInsertPane } from "../search/multiselect.search.insert.pane";

@shortcut()
export class MultiSelectInsertSearcher extends Widget {
    static xtype = "bi.multi_select_insert_searcher";

    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_START = "EVENT_START";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_SEARCHING = "EVENT_SEARCHING";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-select-searcher",
            itemsCreator: emptyFn,
            itemHeight: 24,
            el: {},
            popup: {},
            valueFormatter: emptyFn,
            adapter: null,
            masker: {},
            watermark: i18nText("BI-Basic_Search_And_Patch_Paste"),
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;
        this.editor = createWidget(o.el, {
            type: MultiSelectEditor.xtype,
            watermark: o.watermark,
            height: o.height,
            text: o.text,
            defaultText: o.defaultText,
            listeners: [
                {
                    eventName: MultiSelectEditor.EVENT_FOCUS,
                    action() {
                        self.fireEvent(MultiSelectInsertSearcher.EVENT_FOCUS);
                    },
                },
                {
                    eventName: MultiSelectEditor.EVENT_BLUR,
                    action() {
                        self.fireEvent(MultiSelectInsertSearcher.EVENT_BLUR);
                    },
                }
            ],
        });

        this.searcher = createWidget({
            type: Searcher.xtype,
            element: this,
            height: o.height,
            isAutoSearch: false,
            isAutoSync: false,
            onSearch(op, callback) {
                callback();
            },
            el: this.editor,

            popup: extend(
                {
                    type: MultiSelectSearchInsertPane.xtype,
                    valueFormatter: o.valueFormatter,
                    itemFormatter: o.itemFormatter,
                    keywordGetter() {
                        return self.editor.getKeyword();
                    },
                    itemsCreator(op, callback) {
                        const keyword = self.editor.getKeyword();
                        op.keywords = [keyword];
                        this.setKeyword(keyword);
                        o.itemsCreator(op, function () {
                            if (keyword === self.editor.getKeyword()) {
                                callback(...arguments);
                            }
                        });
                    },
                    itemHeight: o.itemHeight,
                    value: o.value,
                },
                o.popup
            ),

            adapter: o.adapter,
            masker: o.masker,
        });
        this.searcher.on(Searcher.EVENT_START, () => {
            self.fireEvent(MultiSelectInsertSearcher.EVENT_START);
        });
        this.searcher.on(Searcher.EVENT_PAUSE, () => {
            self.fireEvent(MultiSelectInsertSearcher.EVENT_PAUSE);
        });
        this.searcher.on(Searcher.EVENT_STOP, () => {
            self.fireEvent(MultiSelectInsertSearcher.EVENT_STOP);
        });
        this.searcher.on(Searcher.EVENT_CHANGE, function () {
            self.fireEvent(MultiSelectInsertSearcher.EVENT_CHANGE, arguments);
        });
        this.searcher.on(Searcher.EVENT_SEARCHING, function () {
            const keywords = this.getKeywords();
            self.fireEvent(
                MultiSelectInsertSearcher.EVENT_SEARCHING,
                keywords.length > PasteLimit ? keywords.slice(0, PasteLimit).concat([BlankSplitChar]) : keywords.slice(0, PasteLimit)
            );
        });
        if (isNotNull(o.value)) {
            this.setState(o.value);
        }
    }

    adjustView() {
        this.searcher.adjustView();
    }

    isSearching() {
        return this.searcher.isSearching();
    }

    stopSearch() {
        this.searcher.stopSearch();
    }

    getKeywordsLength() {
        const keywords = this.editor.getKeywords();

        return keywords[keywords.length - 1] === BlankSplitChar ? keywords.length - 1 : keywords.length;
    }

    getKeyword() {
        let keywords = this.editor.getKeywords().slice(0, PasteLimit);
        if (keywords[keywords.length - 1] === BlankSplitChar) {
            keywords = keywords.slice(0, keywords.length - 1);
        }

        return isEmptyArray(keywords) ? "" : keywords[keywords.length - 1];
    }

    hasMatched() {
        return this.searcher.hasMatched();
    }

    hasChecked() {
        return this.searcher.getView() && this.searcher.getView().hasChecked();
    }

    setAdapter(adapter) {
        this.searcher.setAdapter(adapter);
    }

    setState(ob) {
        let state;
        const o = this.options;
        ob || (ob = {});
        ob.value || (ob.value = []);
        if (ob.type === Selection.All) {
            if (ob.value.length === 0) {
                this.editor.setState(Selection.All);
            } else if (size(ob.assist) <= 20) {
                state = "";
                each(ob.assist, (i, v) => {
                    if (i === 0) {
                        state += `${v === null ? "" : o.valueFormatter(`${v}`) || v}`;
                    } else {
                        state += `,${v === null ? "" : o.valueFormatter(`${v}`) || v}`;
                    }
                });
                this.editor.setState(state);
            } else {
                this.editor.setState(Selection.Multi);
            }
        } else {
            if (ob.value.length === 0) {
                this.editor.setState(Selection.None);
            } else if (size(ob.value) <= 20) {
                state = "";
                each(ob.value, (i, v) => {
                    if (i === 0) {
                        state += `${v === null ? "" : o.valueFormatter(`${v}`) || v}`;
                    } else {
                        state += `,${v === null ? "" : o.valueFormatter(`${v}`) || v}`;
                    }
                });
                this.editor.setState(state);
            } else {
                this.editor.setState(Selection.Multi);
            }
        }
    }

    getState() {
        return this.editor.getState();
    }

    setValue(ob) {
        this.setState(ob);
        this.searcher.setValue(ob);
    }

    getKey() {
        return this.editor.getValue();
    }

    getValue() {
        return this.searcher.getValue();
    }

    populate(items) {
        this.searcher.populate(...arguments);
    }
}
