import {
    shortcut,
    extend,
    emptyFn,
    createWidget,
    toPix,
    parseFloat,
    HTapeLayout,
    GridLayout,
    isNumeric,
    clamp,
    MIN,
    MAX,
    KeyCode,
    add
} from "@/core";
import { SignEditor } from "@/case";
import { TextEditor } from "../editor";
import { IconButton } from "@/base";
import { Single } from "../../base/single";

@shortcut()
export class NumberEditor extends Single {
    static xtype = "bi.number_editor";

    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig(conf) {
        return extend(super._defaultConfig(...arguments), {
            baseCls: `bi-number-editor bi-focus-shadow ${
                conf.simple ? "bi-border-bottom" : "bi-border bi-border-radius"
            }`,
            validationChecker: emptyFn,
            valueFormatter(v) {
                return v;
            },
            valueParser(v) {
                return v;
            },
            value: 0,
            allowBlank: false,
            errorText: "",
            step: 1,
            min: MIN,
            max: MAX,
            watermark: "",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.editor = createWidget({
            type: SignEditor.xtype,
            height: toPix(o.height, 2),
            simple: o.simple,
            allowBlank: o.allowBlank,
            watermark: o.watermark,
            value: o.valueFormatter(o.value),
            validationChecker: v => {
                // 不设置validationChecker就自动检测
                const parsedValue = o.valueParser(v);
                if (o.validationChecker === emptyFn && !this._checkValueInRange(parsedValue)) {
                    return false;
                }

                return o.validationChecker(parsedValue);
            },
            errorText: o.errorText,
            listeners: [
                {
                    eventName: SignEditor.EVENT_QUICK_DOWN,
                    action: e => {
                        if ([KeyCode.UP, KeyCode.DOWN].includes(e.keyCode)) {
                            e.preventDefault();
                        }
                    },
                },
                {
                    eventName: SignEditor.EVENT_KEY_DOWN,
                    action: keycode => {
                        if (keycode === KeyCode.UP) {
                            this._finetuning(o.step);

                            return;
                        }
                        if (keycode === KeyCode.DOWN) {
                            this._finetuning(-o.step);
                        }
                    },
                }
            ],
        });
        this.editor.on(TextEditor.EVENT_CHANGE, () => {
            //  大多数时候valueFormatter往往需要配合valueParser一起使用
            const value = this.editor.getValue();
            const parsedValue = o.valueParser(value);
            this.editor.setValue(o.valueFormatter(parsedValue));
            this.fireEvent(NumberEditor.EVENT_CHANGE);
        });
        this.editor.on(TextEditor.EVENT_ERROR, () => {
            o.value = parseFloat(this.editor.getLastValidValue());
            this._checkAdjustDisabled(o.value);
            this.element.addClass("error");
        });
        this.editor.on(TextEditor.EVENT_VALID, () => {
            o.value = parseFloat(this.editor.getValue());
            this._checkAdjustDisabled(o.value);
            this.element.removeClass("error");
        });
        this.editor.on(TextEditor.EVENT_CONFIRM, () => {
            this.fireEvent(NumberEditor.EVENT_CONFIRM);
        });
        this.topBtn = createWidget({
            type: IconButton.xtype,
            forceNotSelected: true,
            trigger: "lclick,",
            debounce: false,
            cls: `${
                o.simple ? "solid-triangle-top-font " : "add-up-font bi-border-left "
            }top-button bi-list-item-active2 icon-size-12`,
        });
        this.topBtn.on(IconButton.EVENT_CHANGE, () => {
            this._finetuning(o.step);
            this.fireEvent(NumberEditor.EVENT_CHANGE);
            this.fireEvent(NumberEditor.EVENT_CONFIRM);
        });
        this.bottomBtn = createWidget({
            type: IconButton.xtype,
            trigger: "lclick,",
            forceNotSelected: true,
            debounce: false,
            cls: `${
                o.simple ? "solid-triangle-bottom-font " : "minus-down-font bi-border-left "
            }bottom-button bi-list-item-active2 icon-size-12`,
        });
        this.bottomBtn.on(IconButton.EVENT_CHANGE, () => {
            this._finetuning(-o.step);
            this.fireEvent(NumberEditor.EVENT_CHANGE);
            this.fireEvent(NumberEditor.EVENT_CONFIRM);
        });
        createWidget({
            type: HTapeLayout.xtype,
            height: toPix(o.height, 2),
            element: this,
            items: [
                this.editor,
                {
                    el: {
                        type: GridLayout.xtype,
                        columns: 1,
                        rows: 2,
                        items: [
                            {
                                column: 0,
                                row: 0,
                                el: this.topBtn,
                            },
                            {
                                column: 0,
                                row: 1,
                                el: this.bottomBtn,
                            }
                        ],
                    },
                    width: 23,
                }
            ],
        });
    }

    focus() {
        this.editor.focus();
    }

    isEditing() {
        return this.editor.isEditing();
    }

    _checkValueInRange(v) {
        const o = this.options;

        return !!(isNumeric(v) && parseFloat(v) >= o.min && parseFloat(v) <= o.max);
    }

    _checkAdjustDisabled(v) {
        if (this.options.validationChecker === emptyFn) {
            this.bottomBtn.setEnable(parseFloat(v) > this.options.min);
            this.topBtn.setEnable(parseFloat(v) < this.options.max);
        }
    }

    _finetuning(addValue) {
        const { max, min } = this.options;
        let v = parseFloat(this.getValue());
        v = add(v, addValue);
        v = clamp(v, min, max);
        this.setValue(v);
    }

    setUpEnable(v) {
        this.topBtn.setEnable(!!v);
    }

    setDownEnable(v) {
        this.bottomBtn.setEnable(!!v);
    }

    getLastValidValue() {
        return this.editor.getLastValidValue();
    }

    getLastChangedValue() {
        return this.editor.getLastChangedValue();
    }

    getValue() {
        return this.options.value;
    }

    setValue(v) {
        const o = this.options;
        o.value = v;
        this.editor.setValue(o.valueFormatter(v));
        this._checkAdjustDisabled(o.value);
    }
}
