import {
    shortcut,
    extend,
    emptyFn,
    isKey,
    createWidget,
    toPix,
    isNotNull,
    nextTick,
    AbsoluteLayout,
    makeObject,
    map,
    each,
    remove
} from "@/core";
import { Single, Combo } from "@/base";
import { SingleSelectTrigger } from "./singleselect.trigger";
import { SingleSelectPopupView } from "./singleselect.popup.view";
import { TriggerIconButton } from "@/case";

@shortcut()
export class SingleSelectCombo extends Single {
    static xtype = "bi.single_select_combo";

    static REQ_GET_DATA_LENGTH = 0;
    static REQ_GET_ALL_DATA = -1;
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_SEARCHING = "EVENT_SEARCHING";
    static EVENT_CLICK_ITEM = "EVENT_CLICK_ITEM";
    static EVENT_CONFIRM = "EVENT_CONFIRM";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-combo",
            allowNoSelect: false,
            itemsCreator: emptyFn,
            itemWrapper: emptyFn,
            valueFormatter: emptyFn,
            height: 24,
            allowEdit: true,
        });
    }

    _init() {
        const o = this.options;
        super._init(...arguments);

        const assertShowValue = () => {
            isKey(this._startValue) && (this.storeValue = this._startValue);
            this.trigger.getSearcher().setState(this.storeValue);
        };
        this.storeValue = o.value;
        // 标记正在请求数据
        this.requesting = false;

        this.trigger = createWidget({
            type: SingleSelectTrigger.xtype,
            height: toPix(o.height, o.simple ? 1 : 2),
            // adapter: this.popup,
            allowNoSelect: o.allowNoSelect,
            allowEdit: o.allowEdit,
            valueFormatter: o.valueFormatter,
            itemsCreator: (op, callback) => {
                o.itemsCreator(op, (...args) => {
                    if (op.times === 1 && isNotNull(op.keywords)) {
                        // 预防trigger内部把当前的storeValue改掉
                        this.trigger.setValue(this.getValue());
                    }
                    callback.apply(this, args);
                });
            },
            text: o.text,
            value: this.storeValue,
        });

        this.trigger.on(SingleSelectTrigger.EVENT_FOCUS, () => {
            this.fireEvent(SingleSelectCombo.EVENT_FOCUS);
        });
        this.trigger.on(SingleSelectTrigger.EVENT_BLUR, () => {
            this.fireEvent(SingleSelectCombo.EVENT_BLUR);
        });

        this.trigger.on(SingleSelectTrigger.EVENT_START, () => {
            this._setStartValue();
            this.trigger.getSearcher().setValue(this.storeValue);
        });
        this.trigger.on(SingleSelectTrigger.EVENT_STOP, () => {
            this._setStartValue();
            this.fireEvent(SingleSelectCombo.EVENT_STOP);
        });
        this.trigger.on(SingleSelectTrigger.EVENT_SEARCHING, () => {
            this._dataChange = true;
            this.fireEvent(SingleSelectCombo.EVENT_SEARCHING);
        });

        this.trigger.on(SingleSelectTrigger.EVENT_CHANGE, (value, obj) => {
            this.storeValue = this.trigger.getValue();
            assertShowValue();
            this._defaultState();
            this._dataChange = true;
        });
        this.trigger.on(SingleSelectTrigger.EVENT_COUNTER_CLICK, () => {
            if (!this.combo.isViewVisible()) {
                this.combo.showView();
            }
        });

        this.combo = createWidget({
            type: Combo.xtype,
            cls: o.simple ? "bi-border-bottom" : "bi-border bi-border-radius",
            container: o.container,
            toggle: false,
            el: this.trigger,
            adjustLength: 1,
            popup: {
                type: SingleSelectPopupView.xtype,
                allowNoSelect: o.allowNoSelect,
                ref: _ref => {
                    this.popup = _ref;
                    this.trigger.setAdapter(_ref);
                },
                listeners: [
                    {
                        eventName: SingleSelectPopupView.EVENT_CHANGE,
                        action: () => {
                            this._dataChange = true;
                            this.storeValue = this.popup.getValue();
                            this._adjust(() => {
                                assertShowValue();
                                this._defaultState();
                            });
                            this.fireEvent(SingleSelectCombo.EVENT_CLICK_ITEM);
                        },
                    }
                ],
                itemsCreator: o.itemsCreator,
                itemWrapper: o.itemWrapper,
                valueFormatter: o.valueFormatter,
                onLoaded: () => {
                    nextTick(() => {
                        this.combo.adjustWidth();
                        this.combo.adjustHeight();
                        this.trigger.getSearcher().adjustView();
                    });
                },
            },
            hideChecker(e) {
                return triggerBtn.element.find(e.target).length === 0;
            },
            value: o.value,
        });

        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, () => {
            if (!this.combo.isViewVisible()) {
                this._dataChange = false; // 标记数据是否发生变化
            }
            this.setValue(this.storeValue);
            nextTick(() => {
                this.populate();
            });
        });
        // 当退出的时候如果还在处理请求，则等请求结束后再对外发确定事件
        this.wants2Quit = false;
        this.combo.on(Combo.EVENT_AFTER_HIDEVIEW, () => {
            // important:关闭弹出时又可能没有退出编辑状态
            this.trigger.stopEditing();
            if (this.requesting === true) {
                this.wants2Quit = true;
            } else {
                this._dataChange && this.fireEvent(SingleSelectCombo.EVENT_CONFIRM);
            }
        });

        const triggerBtn = createWidget({
            type: TriggerIconButton.xtype,
            width: o.height,
            height: o.height,
            cls: "single-select-trigger-icon-button",
        });
        triggerBtn.on(TriggerIconButton.EVENT_CHANGE, () => {
            if (this.combo.isViewVisible()) {
                this.combo.hideView();
            } else {
                this.combo.showView();
            }
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.combo,
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                },
                {
                    el: triggerBtn,
                    right: 0,
                    top: 0,
                    bottom: 0,
                }
            ],
        });
    }

    _defaultState() {
        this.trigger.stopEditing();
        this.combo.hideView();
    }

    _assertValue(val) {}

    _makeMap(values) {
        return makeObject(values || []);
    }

    _joinKeywords(keywords, callback) {
        const o = this.options;
        this._assertValue(this.storeValue);
        this.requesting = true;
        o.itemsCreator(
            {
                type: SingleSelectCombo.REQ_GET_ALL_DATA,
                keywords,
            },
            ob => {
                const values = map(ob.items, "value");
                digest(values);
            }
        );

        const digest = items => {
            const selectedMap = this._makeMap(items);
            each(keywords, (i, val) => {
                if (isNotNull(selectedMap[val])) {
                    remove(this.storeValue.value, val);
                }
            });
            this._adjust(callback);
        };
    }

    _adjust(callback) {
        const adjust = () => {
            if (this.wants2Quit === true) {
                this._dataChange && this.fireEvent(SingleSelectCombo.EVENT_CONFIRM);
                this.wants2Quit = false;
            }
            this.requesting = false;
        };

        const o = this.options;
        if (!this._count) {
            o.itemsCreator(
                {
                    type: SingleSelectCombo.REQ_GET_DATA_LENGTH,
                },
                res => {
                    this._count = res.count;
                    adjust();
                    callback();
                }
            );
        } else {
            adjust();
            callback();
        }
    }

    _setStartValue(value) {
        this._startValue = value;
        this.popup.setStartValue(value);
    }

    setValue(v) {
        this.storeValue = v;
        this._assertValue(this.storeValue);
        this.combo.setValue(this.storeValue);
    }

    getValue() {
        return this.storeValue;
    }

    populate() {
        this._count = null;
        this.combo.populate(...arguments);
    }
}
