import { SingleSelectSearchPane } from "../search";
import { shortcut, Widget, extend, emptyFn, createWidget, isNotNull, isUndefined, Selection } from "@/core";
import { SingleSelectEditor } from "./editor.singleselect";
import { Searcher } from "@/base";

@shortcut()
export class SingleSelectSearcher extends Widget {
    static xtype = "bi.single_select_searcher";

    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_START = "EVENT_START";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_SEARCHING = "EVENT_SEARCHING";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-searcher",
            itemsCreator: emptyFn,
            el: {},
            popup: {},
            valueFormatter: emptyFn,
            adapter: null,
            masker: {},
            allowNoSelect: false,
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;
        this.editor = createWidget(o.el, {
            type: SingleSelectEditor.xtype,
            height: o.height,
            watermark: o.watermark,
            text: o.text,
            listeners: [
                {
                    eventName: SingleSelectEditor.EVENT_FOCUS,
                    action: () => {
                        this.fireEvent(SingleSelectSearcher.EVENT_FOCUS);
                    },
                },
                {
                    eventName: SingleSelectEditor.EVENT_BLUR,
                    action: () => {
                        this.fireEvent(SingleSelectSearcher.EVENT_BLUR);
                    },
                }
            ],
        });

        this.searcher = createWidget({
            type: Searcher.xtype,
            element: this,
            height: o.height,
            isAutoSearch: false,
            isAutoSync: false,
            onSearch(op, callback) {
                callback();
            },
            el: this.editor,

            popup: extend(
                {
                    type: SingleSelectSearchPane.xtype,
                    allowNoSelect: o.allowNoSelect,
                    valueFormatter: o.valueFormatter,
                    keywordGetter: () => this.editor.getValue(),
                    itemsCreator(op, callback) {
                        const keyword = self.editor.getValue();
                        op.keywords = [keyword];
                        this.setKeyword(keyword);
                        o.itemsCreator(op, callback);
                    },
                    value: o.value,
                },
                o.popup
            ),

            adapter: o.adapter,
            masker: o.masker,
        });
        this.searcher.on(Searcher.EVENT_START, () => {
            this.fireEvent(SingleSelectSearcher.EVENT_START);
        });
        this.searcher.on(Searcher.EVENT_PAUSE, () => {
            this.fireEvent(SingleSelectSearcher.EVENT_PAUSE);
        });
        this.searcher.on(Searcher.EVENT_STOP, () => {
            this.fireEvent(SingleSelectSearcher.EVENT_STOP);
        });
        this.searcher.on(Searcher.EVENT_CHANGE, (...args) => {
            this.fireEvent(SingleSelectSearcher.EVENT_CHANGE, ...args);
        });
        this.searcher.on(Searcher.EVENT_SEARCHING, () => {
            const keywords = this.searcher.getKeywords();
            this.fireEvent(SingleSelectSearcher.EVENT_SEARCHING, keywords);
        });

        if (isNotNull(o.value)) {
            this.setState(o.value);
        }
    }

    adjustView() {
        this.searcher.adjustView();
    }

    isSearching() {
        return this.searcher.isSearching();
    }

    stopSearch() {
        this.searcher.stopSearch();
    }

    getKeyword() {
        return this.editor.getKeyword();
    }

    hasMatched() {
        return this.searcher.hasMatched();
    }

    hasChecked() {
        return this.searcher.getView() && this.searcher.getView().hasChecked();
    }

    setAdapter(adapter) {
        this.searcher.setAdapter(adapter);
    }

    setState(v) {
        const o = this.options;
        if (isUndefined(v)) {
            this.editor.setState(Selection.None);
        } else {
            v = v ?? "";
            this.editor.setState(o.valueFormatter(`${v}`) || `${v}`);
        }
    }

    setValue(ob) {
        this.setState(ob);
        this.searcher.setValue(ob);
    }

    getKey() {
        return this.editor.getValue();
    }

    getValue() {
        return this.searcher.getValue();
    }

    populate(items) {
        this.searcher.populate(...arguments);
    }
}
