import {
    shortcut,
    extend,
    i18nText,
    bind,
    createWidget,
    isPositiveInteger,
    checkDateVoid,
    parseDateTime,
    isNotNull,
    isNotEmptyString,
    parseInt,
    print,
    getDate,
    HorizontalFillLayout,
    isEmptyString
} from "@/core";
import { Trigger, TextButton } from "@/base";
import { SignEditor, TriggerIconButton } from "@/case";
import { DynamicDateCombo } from "@/widget/dynamicdate/dynamicdate.combo";
import { DynamicDateHelper } from "@/widget/dynamicdate/dynamicdate.caculate";

@shortcut()
export class DynamicYearTrigger extends Trigger {
    static xtype = "bi.dynamic_year_trigger";

    _const = { hgap: 4, vgap: 2, iconWidth: 24 };

    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_START = "EVENT_START";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_VALID = "EVENT_VALID";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-year-trigger",
            min: "1900-01-01", // 最小日期
            max: "2099-12-31", // 最大日期
            height: 24,
            watermark: i18nText("BI-Basic_Unrestricted"),
        });
    }

    beforeInit(callback) {
        const o = this.options;
        o.title = bind(this._titleCreator, this);
        callback();
    }

    _init() {
        super._init(...arguments);
        const o = this.options,
            c = this._const;
        this.editor = createWidget({
            type: SignEditor.xtype,
            simple: o.simple,
            height: o.height,
            validationChecker: v => v === "" || (isPositiveInteger(v) && !checkDateVoid(v, 1, 1, o.min, o.max)[0]),
            quitChecker: () => false,
            hgap: c.hgap,
            vgap: c.vgap,
            watermark: o.watermark,
            allowBlank: true,
            errorText: v => {
                if (isPositiveInteger(v)) {
                    const start = parseDateTime(o.min, "%Y-%X-%d");
                    const end = parseDateTime(o.max, "%Y-%X-%d");

                    return i18nText("BI-Basic_Year_Range_Error", start.getFullYear(), end.getFullYear());
                }

                return i18nText("BI-Year_Trigger_Invalid_Text");
            },
        });
        this.editor.on(SignEditor.EVENT_KEY_DOWN, () => {
            this.fireEvent(DynamicYearTrigger.EVENT_KEY_DOWN, ...arguments);
        });
        this.editor.on(SignEditor.EVENT_FOCUS, () => {
            this.fireEvent(DynamicYearTrigger.EVENT_FOCUS);
        });
        this.editor.on(SignEditor.EVENT_STOP, () => {
            this.fireEvent(DynamicYearTrigger.EVENT_STOP);
        });
        this.editor.on(SignEditor.EVENT_CONFIRM, () => {
            const value = this.editor.getValue();
            if (isNotNull(value)) {
                this.editor.setValue(value);
            }
            if (isNotEmptyString(value)) {
                this.storeValue = {
                    type: DynamicDateCombo.Static,
                    value: {
                        year: value,
                    },
                };
            }

            this.fireEvent(DynamicYearTrigger.EVENT_CONFIRM);
        });
        this.editor.on(SignEditor.EVENT_SPACE, () => {
            if (this.editor.isValid()) {
                this.editor.blur();
            }
        });
        this.editor.on(SignEditor.EVENT_START, () => {
            this.fireEvent(DynamicYearTrigger.EVENT_START);
        });
        this.editor.on(SignEditor.EVENT_ERROR, () => {
            this.fireEvent(DynamicYearTrigger.EVENT_ERROR);
        });
        this.editor.on(SignEditor.EVENT_VALID, () => {
            this.fireEvent(DynamicYearTrigger.EVENT_VALID);
        });
        createWidget({
            element: this,
            type: HorizontalFillLayout.xtype,
            columnSize: ["fill", "", ""],
            items: [
                {
                    el: this.editor,
                },
                {
                    el: {
                        type: TextButton.xtype,
                        baseCls: "bi-trigger-year-text",
                        text: i18nText("BI-Multi_Date_Year"),
                    },
                },
                {
                    el: {
                        type: TriggerIconButton.xtype,
                        width: this._const.iconWidth,
                    },
                }
            ],
        });
        this.setValue(o.value);
    }

    _getText(obj) {
        let value = "";
        if (isNotNull(obj.year) && parseInt(obj.year) !== 0) {
            value +=
                Math.abs(obj.year) +
                i18nText("BI-Basic_Year") +
                (obj.year < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
        }

        return value;
    }

    _setInnerValue(date) {
        const dateStr = print(date, "%Y");
        this.editor.setState(dateStr);
        this.editor.setValue(dateStr);
    }

    _titleCreator() {
        const storeValue = this.storeValue || {};
        const type = storeValue.type || DynamicDateCombo.Static;
        let value = storeValue.value;
        if (!this.editor.isValid()) {
            return "";
        }

        switch (type) {
            case DynamicDateCombo.Dynamic: {
                const text = this._getText(value);
                let date = getDate();
                date = DynamicDateHelper.getCalculation(value);
                const dateStr = print(date, "%Y");

                return isEmptyString(text) ? dateStr : `${text}:${dateStr}`;
            }

            case DynamicDateCombo.Static:
            default:
                value = value || {};

                return value.year;
        }
    }

    setValue(v) {
        let type, value;
        let date = getDate();
        this.storeValue = v;
        if (isNotNull(v)) {
            type = v.type || DynamicDateCombo.Static;
            value = v.value || v;
        }
        switch (type) {
            case DynamicDateCombo.Dynamic: {
                const text = this._getText(value);
                date = DynamicDateHelper.getCalculation(value);
                this._setInnerValue(date, text);
                break;
            }
            case DynamicDateCombo.Static:
            default:
                value = value || {};
                this.editor.setState(value.year);
                this.editor.setValue(value.year);
                break;
        }
    }

    setMinDate(minDate) {
        if (isNotEmptyString(this.options.min)) {
            this.options.min = minDate;
        }
    }

    setMaxDate(maxDate) {
        if (isNotEmptyString(this.options.max)) {
            this.options.max = maxDate;
        }
    }

    getValue() {
        return this.storeValue;
    }

    getKey() {
        return this.editor.getValue() | 0;
    }

    setWaterMark(v) {
        this.editor.setWaterMark(v);
    }
}
