import {
    shortcut,
    HorizontalFillLayout,
    createWidget,
    i18nText,
    print,
    parseDateTime,
    checkDateVoid,
    isNotNull,
    checkDateLegal
} from "@/core";
import { Single, Label, Bubbles } from "@/base";
import { DynamicYearCombo } from "../year";

@shortcut()
export class YearInterval extends Single {
    static xtype = "bi.year_interval";

    constants = {
        height: 24,
        width: 25,
        lgap: 15,
        offset: -15,
        timeErrorCls: "time-error",
    };
    props = {
        extraCls: "bi-year-interval",
        minDate: "1900-01-01",
        maxDate: "2099-12-31",
        supportDynamic: true,
        simple: false,
    };

    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    render() {
        const o = this.options;

        o.value = o.value || {};
        this.left = this._createCombo(o.value.start, o.watermark?.start);
        this.right = this._createCombo(o.value.end, o.watermark?.end);

        return {
            type: HorizontalFillLayout.xtype,
            columnSize: ["fill", "", "fill"],
            items: [
                {
                    el: this.left,
                },
                {
                    el: {
                        type: Label.xtype,
                        height: o.height,
                        hgap: 5,
                        text: "-",
                        ref: _ref => {
                            this.label = _ref;
                        },
                    },
                },
                {
                    el: this.right,
                }
            ],
        };
    }

    _createCombo(v, watermark) {
        const o = this.options;
        const combo = createWidget({
            type: DynamicYearCombo.xtype,
            simple: o.simple,
            supportDynamic: o.supportDynamic,
            minDate: o.minDate,
            maxDate: o.maxDate,
            height: o.height,
            behaviors: o.behaviors,
            value: v,
            watermark,
            listeners: [
                {
                    eventName: DynamicYearCombo.EVENT_BEFORE_POPUPVIEW,
                    action: () => {
                        this.fireEvent(YearInterval.EVENT_BEFORE_POPUPVIEW);
                    },
                }
            ],
        });
        combo.on(DynamicYearCombo.EVENT_ERROR, () => {
            this._clearTitle();
            Bubbles.hide("error");
            this.element.removeClass(this.constants.timeErrorCls);
            this.fireEvent(YearInterval.EVENT_ERROR);
        });

        combo.on(DynamicYearCombo.EVENT_VALID, () => {
            this._checkValid();
        });

        combo.on(DynamicYearCombo.EVENT_FOCUS, () => {
            this._checkValid();
        });

        combo.on(DynamicYearCombo.EVENT_CONFIRM, () => {
            Bubbles.hide("error");
            const smallDate = this.left.getKey(),
                bigDate = this.right.getKey();
            if (
                this.left.isStateValid() &&
                this.right.isStateValid() &&
                this._check(smallDate, bigDate) &&
                this._compare(smallDate, bigDate)
            ) {
                this._setTitle(i18nText("BI-Time_Interval_Error_Text"));
                this.element.addClass(this.constants.timeErrorCls);
                this.fireEvent(YearInterval.EVENT_ERROR);
            } else {
                this._clearTitle();
                this.element.removeClass(this.constants.timeErrorCls);
                this.fireEvent(YearInterval.EVENT_CHANGE);
            }
        });

        return combo;
    }

    _dateCheck(date) {
        return print(parseDateTime(date, "%Y"), "%Y") === date || print(parseDateTime(date, "%Y"), "%Y") === date;
    }

    _checkVoid(obj) {
        const o = this.options;

        return !checkDateVoid(obj.year, 1, 1, o.minDate, o.maxDate)[0];
    }

    _check(smallDate, bigDate) {
        const smallObj = smallDate.match(/\d+/g),
            bigObj = bigDate.match(/\d+/g);

        let smallDate4Check = "";
        if (isNotNull(smallObj)) {
            smallDate4Check = smallObj[0] || "";
        }

        let bigDate4Check = "";
        if (isNotNull(bigObj)) {
            bigDate4Check = bigObj[0] || "";
        }

        return (
            this._dateCheck(smallDate4Check) &&
            checkDateLegal(smallDate4Check) &&
            this._checkVoid({
                year: smallObj[0],
                month: 1,
                day: 1,
            }) &&
            this._dateCheck(bigDate4Check) &&
            checkDateLegal(bigDate4Check) &&
            this._checkVoid({
                year: bigObj[0],
                month: 12,
                day: 1,
            })
        );
    }

    _compare(smallDate, bigDate) {
        smallDate = print(parseDateTime(smallDate, "%Y"), "%Y");
        bigDate = print(parseDateTime(bigDate, "%Y"), "%Y");

        return isNotNull(smallDate) && isNotNull(bigDate) && smallDate > bigDate;
    }

    _setTitle(v) {
        this.setTitle(v);
    }

    _clearTitle() {
        this.setTitle("");
    }

    _checkValid() {
        Bubbles.hide("error");
        const smallDate = this.left.getKey(),
            bigDate = this.right.getKey();
        if (
            this.left.isValid() &&
            this.right.isValid() &&
            this._check(smallDate, bigDate) &&
            this._compare(smallDate, bigDate)
        ) {
            this._setTitle(i18nText("BI-Time_Interval_Error_Text"));
            this.element.addClass(this.constants.timeErrorCls);
            Bubbles.show("error", i18nText("BI-Time_Interval_Error_Text"), this, {
                offsetStyle: "center",
            });
            this.fireEvent(YearInterval.EVENT_ERROR);
        } else {
            this._clearTitle();
            this.element.removeClass(this.constants.timeErrorCls);
        }
    }

    setMinDate(minDate) {
        const o = this.options;
        o.minDate = minDate;
        this.left.setMinDate(minDate);
        this.right.setMinDate(minDate);
    }

    setMaxDate(maxDate) {
        const o = this.options;
        o.maxDate = maxDate;
        this.left.setMaxDate(maxDate);
        this.right.setMaxDate(maxDate);
    }

    setValue(date) {
        date = date || {};
        this.left.setValue(date.start);
        this.right.setValue(date.end);

        this._checkValid();
    }

    getValue() {
        return { start: this.left.getValue(), end: this.right.getValue() };
    }
}
