import {
    shortcut,
    Widget,
    i18nText,
    bind,
    VerticalLayout,
    parseDateTime,
    extend,
    checkDateVoid,
    isNotEmptyString,
    SIZE_CONSANTS
} from "@/core";
import { DynamicDateCard, DynamicDateParamItem, DynamicDateHelper } from "../dynamicdate";
import { Label, Bubbles } from "@/base";

@shortcut()
export class DynamicYearMonthCard extends Widget {
    static xtype = "bi.dynamic_year_month_card";

    static EVENT_CHANGE = "EVENT_CHANGE";

    props = { baseCls: "bi-year-month-card" };

    render() {
        return {
            type: VerticalLayout.xtype,
            items: [
                {
                    type: Label.xtype,
                    text: i18nText("BI-Multi_Date_Relative_Current_Time"),
                    textAlign: "left",
                    height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
                },
                {
                    type: VerticalLayout.xtype,
                    ref: _ref => {
                        this.wrapper = _ref;
                    },
                    items: [
                        {
                            el: {
                                type: DynamicDateParamItem.xtype,
                                validationChecker: bind(this._checkDate, this),
                                ref: _ref => {
                                    this.year = _ref;
                                },
                                listeners: [
                                    {
                                        eventName: "EVENT_CHANGE",
                                        action: () => {
                                            this.fireEvent("EVENT_CHANGE");
                                        },
                                    },
                                    {
                                        eventName: "EVENT_INPUT_CHANGE",
                                        action: () => {
                                            Bubbles.hide("dynamic-year-month-error");
                                        },
                                    }
                                ],
                            },
                            bgap: 10,
                        },
                        {
                            type: DynamicDateParamItem.xtype,
                            dateType: DynamicDateCard.TYPE.MONTH,
                            ref: _ref => {
                                this.month = _ref;
                            },
                            listeners: [
                                {
                                    eventName: "EVENT_CHANGE",
                                    action: () => {
                                        this.fireEvent("EVENT_CHANGE");
                                    },
                                },
                                {
                                    eventName: "EVENT_INPUT_CHANGE",
                                    action: () => {
                                        Bubbles.hide("dynamic-year-month-error");
                                    },
                                }
                            ],
                        }
                    ],
                }
            ],
            vgap: 10,
            hgap: 10,
        };
    }

    _getErrorText() {
        const o = this.options;
        const start = parseDateTime(o.min, "%Y-%X-%d");
        const end = parseDateTime(o.max, "%Y-%X-%d");

        return i18nText(
            "BI-Basic_Year_Month_Range_Error",
            start.getFullYear(),
            start.getMonth() + 1,
            end.getFullYear(),
            end.getMonth() + 1
        );
    }

    _checkDate(obj) {
        const o = this.options;
        const date = DynamicDateHelper.getCalculation(extend(this._getValue(), this._digestDateTypeValue(obj)));

        return !checkDateVoid(date.getFullYear(), date.getMonth() + 1, date.getDate(), o.min, o.max)[0];
    }

    _digestDateTypeValue(value) {
        const valueMap = {};
        switch (value.dateType) {
            case DynamicDateCard.TYPE.YEAR:
                valueMap.year = value.offset === 0 ? -value.value : +value.value;
                break;
            case DynamicDateCard.TYPE.MONTH:
                valueMap.month = value.offset === 0 ? -value.value : +value.value;
                break;
            default:
                break;
        }

        return valueMap;
    }

    _createValue(type, v) {
        return {
            dateType: type,
            value: Math.abs(v),
            offset: v > 0 ? 1 : 0,
        };
    }

    setMinDate(minDate) {
        if (isNotEmptyString(this.options.min)) {
            this.options.min = minDate;
        }
    }

    setMaxDate(maxDate) {
        if (isNotEmptyString(this.options.max)) {
            this.options.max = maxDate;
        }
    }

    setValue(v) {
        v = v || { year: 0, month: 0 };
        this.year.setValue(this._createValue(DynamicDateCard.TYPE.YEAR, v.year));
        this.month.setValue(this._createValue(DynamicDateCard.TYPE.MONTH, v.month));
    }

    _getValue() {
        const year = this.year.getValue();
        const month = this.month.getValue();

        return {
            year: year.offset === 0 ? -year.value : year.value,
            month: month.offset === 0 ? -month.value : month.value,
        };
    }

    getInputValue() {
        return this._getValue();
    }

    getValue() {
        return this.checkValidation() ? this._getValue() : {};
    }

    checkValidation(show) {
        let errorText;
        const yearInvalid = !this.year.checkValidation();
        const monthInvalid = !this.month.checkValidation();
        let invalid = yearInvalid || monthInvalid;
        if (invalid) {
            errorText = i18nText("BI-Please_Input_Natural_Number");
        } else {
            invalid = !this._checkDate(this._getValue());
            errorText = this._getErrorText();
        }
        invalid && show && Bubbles.show("dynamic-year-month-error", errorText, this.wrapper);

        return !invalid;
    }
}
